@@ -16,26 +16,34 @@
  */
 package org.apache.activemq.artemis.cli.commands.queue;
 
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.TreeMap;
 
-import io.airlift.airline.Command;
-import io.airlift.airline.Option;
 import org.apache.activemq.artemis.api.core.JsonUtil;
-import org.apache.activemq.artemis.api.core.client.ClientMessage;
 import org.apache.activemq.artemis.api.core.management.ManagementHelper;
-import org.apache.activemq.artemis.cli.commands.AbstractAction;
+import org.apache.activemq.artemis.api.core.management.SimpleManagement;
+import org.apache.activemq.artemis.cli.Terminal;
 import org.apache.activemq.artemis.cli.commands.ActionContext;
+import org.apache.activemq.artemis.cli.commands.messages.ConnectionAbstract;
 import org.apache.activemq.artemis.json.JsonArray;
 import org.apache.activemq.artemis.json.JsonObject;
+import org.apache.activemq.artemis.utils.TableOut;
+import picocli.CommandLine.Command;
+import picocli.CommandLine.Option;
 
-@Command(name = "stat", description = "prints out basic stats associated with queues. Output includes CONSUMER_COUNT (number of consumers), MESSAGE_COUNT (current message count on the queue, including scheduled, paged and in-delivery messages), MESSAGES_ADDED (messages added to the queue), DELIVERING_COUNT (messages broker is currently delivering to consumer(s)), MESSAGES_ACKED (messages acknowledged from the consumer(s))." + " Queues can be filtered using EITHER '--queueName X' where X is contained in the queue name OR using a full filter '--field NAME --operation EQUALS --value X'.")
-public class StatQueue extends AbstractAction {
+@Command(name = "stat", description = "Print basic stats of a queue. Output includes CONSUMER_COUNT (number of consumers), MESSAGE_COUNT (current message count on the queue, including scheduled, paged and in-delivery messages), MESSAGES_ADDED (messages added to the queue), DELIVERING_COUNT (messages broker is currently delivering to consumer(s)), MESSAGES_ACKED (messages acknowledged from the consumer(s))." + " Queues can be filtered using EITHER '--queueName X' where X is contained in the queue name OR using a full filter '--field NAME --operation EQUALS --value X'.")
+public class StatQueue extends ConnectionAbstract {
+
+   private static final String NOT_APPLICABLE = "";
+
+   private static final String MANAGEMENT_QUEUE = "activemq.management";
 
    public enum FIELD {
-      NAME("name"), ADDRESS("address"), CONSUMER_COUNT("consumerCount"), MESSAGE_COUNT("messageCount"), MESSAGES_ADDED("messagesAdded"), DELIVERING_COUNT("deliveringCount"), MESSAGES_ACKED("messagesAcked"), SCHEDULED_COUNT("scheduledCount"), ROUTING_TYPE("routingType");
+      NAME("name", false), ADDRESS("address", false), CONSUMER_COUNT("consumerCount", true), MESSAGE_COUNT("messageCount", true), MESSAGES_ADDED("messagesAdded", true), DELIVERING_COUNT("deliveringCount", true), MESSAGES_ACKED("messagesAcked", true), SCHEDULED_COUNT("scheduledCount", true), ROUTING_TYPE("routingType", true), INTERNAL("internalQueue", true);
 
       private static final Map<String, FIELD> lookup = new TreeMap<>(String.CASE_INSENSITIVE_ORDER);
 
@@ -46,9 +54,11 @@ public class StatQueue extends AbstractAction {
       }
 
       private String jsonId;
+      private boolean center;
 
-      FIELD(String jsonId) {
+      FIELD(String jsonId, boolean center) {
          this.jsonId = jsonId;
+         this.center = center;
       }
 
       String getJsonId() {
@@ -68,24 +78,52 @@ public class StatQueue extends AbstractAction {
 
    public static final int DEFAULT_MAX_COLUMN_SIZE = 25;
 
-   @Option(name = "--queueName", description = "display queue stats for queue(s) with names containing this string.")
+   @Option(names = "--queueName", description = "Display queue stats for queue(s) with names containing this string.")
    private String queueName;
 
-   @Option(name = "--field", description = "field to use in filter. Possible values NAME, ADDRESS, MESSAGE_COUNT, MESSAGES_ADDED, DELIVERING_COUNT, MESSAGES_ACKED, SCHEDULED_COUNT, ROUTING_TYPE.")
+   @Option(names = "--field", description = "The field to filter. Possible values: NAME, ADDRESS, MESSAGE_COUNT, MESSAGES_ADDED, DELIVERING_COUNT, MESSAGES_ACKED, SCHEDULED_COUNT, ROUTING_TYPE.")
    private String fieldName;
 
-   @Option(name = "--operation", description = "operation to use in filter. Possible values CONTAINS, NOT_CONTAINS, EQUALS, GREATER_THAN, LESS_THAN.")
+   @Option(names = "--operation", description = "The operation to filter. Possible values: CONTAINS, NOT_CONTAINS, EQUALS, GREATER_THAN, LESS_THAN.")
    private String operationName;
 
-   @Option(name = "--value", description = "value to use in the filter.")
+   @Option(names = "--value", description = "The value to filter.")
    private String value;
 
-   @Option(name = "--maxRows", description = "max number of queues displayed. Default is 50.")
+   @Option(names = "--maxRows", description = "The max number of queues displayed. Default is 50.")
    private int maxRows = DEFAULT_MAX_ROWS;
 
-   @Option(name = "--maxColumnSize", description = "max width of data column. Set to -1 for no limit. Default is 25.")
+   @Option(names = "--maxColumnSize", description = "The max width of data column. Set to -1 for no limit. Default is 25.")
    private int maxColumnSize = DEFAULT_MAX_COLUMN_SIZE;
 
+   @Option(names = "--clustered", description = "Expands the report for all nodes on the topology")
+   private boolean clustered = false;
+
+   @Option(names = "--include-management", description = "Include queues created for notification management in the output")
+   private boolean includeManagement = false;
+
+   @Option(names = "--loop", description = "Keep Queue Stat in a forever loop, that you can interrupt with Ctrl-C, sleeping for --loop-time between each iteration.")
+   private boolean useLoop = false;
+
+   private static final long DEFAULT_SLEEP = 60_000;
+   @Option(names = "--loop-sleep", description = "Amount of Milliseconds to sleep before each iteration on queue stat. Default=60000", hidden = true)
+   private long oldSleep = -1;
+
+   @Option(names = "--sleep", description = "Amount of Milliseconds to sleep before each iteration on queue stat. Default=60000")
+   private long loopSleep = -1;
+
+   @Option(names = "--single-line-header", description = "Use a single line on the header titles")
+   private boolean singleLineHeader = false;
+
+   public boolean isSingleLineHeader() {
+      return singleLineHeader;
+   }
+
+   public StatQueue setSingleLineHeader(boolean singleLineHeader) {
+      this.singleLineHeader = singleLineHeader;
+      return this;
+   }
+
    private int statCount = 0;
 
    //easier for testing
@@ -147,28 +185,81 @@ public class StatQueue extends AbstractAction {
          getActionContext().out.println("filter is '" + filter + "'");
          getActionContext().out.println("maxRows='" + maxRows + "'");
       }
-      printStats(getActionContext(), filter);
+      createConnectionFactory();
+
+      singleExeuction(context, filter);
+
+      // if using the hidden parameter
+      if (oldSleep != -1) {
+         loopSleep = oldSleep;
+      }
+
+      if (loopSleep != -1) {
+         // if --loop-sleep was passed as an argument, it is assumed the user also meant --loop
+         useLoop = true;
+      }
+
+      if (useLoop && loopSleep == -1) {
+         loopSleep = DEFAULT_SLEEP;
+      }
+
+      while (useLoop) {
+         getActionContext().out.println("Waiting " + loopSleep + " before another queue stat iteration");
+         Thread.sleep(loopSleep);
+         getActionContext().out.println(new Date() + ">> Queue stat results for " + getBrokerInstance());
+         try {
+            singleExeuction(context, filter);
+         } catch (Throwable e) {
+            e.printStackTrace(getActionContext().err);
+         }
+      }
+
       return statCount;
    }
 
-   private void printStats(final ActionContext context, final String filter) throws Exception {
-      performCoreManagement(new ManagementCallback<ClientMessage>() {
-         @Override
-         public void setUpInvocation(ClientMessage message) throws Exception {
-            ManagementHelper.putOperationInvocation(message, "broker", "listQueues", filter, 1, maxRows);
-         }
+   private void singleExeuction(ActionContext context, String filter) throws Exception {
+      try (SimpleManagement simpleManagement = new SimpleManagement(brokerURL, user, password).open()) {
+         String nodeID = simpleManagement.getNodeID();
+         JsonArray topology = simpleManagement.listNetworkTopology();
 
-         @Override
-         public void requestSuccessful(ClientMessage reply) throws Exception {
-            final String result = (String) ManagementHelper.getResult(reply, String.class);
-            printStats(result);
-         }
+         if (clustered && topology.size() > 1) {
+            context.out.println(Terminal.YELLOW_UNICODE + "*******************************************************************************************************************************");
+            context.out.println(">>> Queue stats on node " + nodeID + ", url=" + brokerURL + Terminal.CLEAR_UNICODE);
+            printStats(brokerURL, filter);
 
-         @Override
-         public void requestFailed(ClientMessage reply) throws Exception {
-            String errMsg = (String) ManagementHelper.getResult(reply, String.class);
-            getActionContext().err.println("Failed to get Stats for Queues. Reason: " + errMsg);
+            for (int i = 0; i < topology.size(); i++) {
+               JsonObject node = topology.getJsonObject(i);
+               if (node.getString("nodeID").equals(nodeID) || node.getJsonString("live") == null) {
+                  continue;
+               }
+
+               String url = "tcp://" + node.getString("live");
+
+               context.out.println(Terminal.YELLOW_UNICODE + "*******************************************************************************************************************************");
+               context.out.println(">>> Queue stats on node " + node.getString("nodeID") + ", url=" + url + Terminal.CLEAR_UNICODE);
+
+               printStats(url, filter);
+            }
+         } else {
+            printStats(brokerURL, filter);
+            if (topology.size() > 1) {
+               context.out.println();
+               context.out.println("Note: Use " + Terminal.RED_UNICODE + "--clustered" + Terminal.CLEAR_UNICODE + " to expand the report to other nodes in the topology.");
+               context.out.println();
+            }
          }
+      }
+   }
+
+   private void printStats(String uri, final String filter) throws Exception {
+      performCoreManagement(uri, user, password, message -> {
+         ManagementHelper.putOperationInvocation(message, "broker", "listQueues", filter, 1, maxRows);
+      }, reply -> {
+         final String result = (String) ManagementHelper.getResult(reply, String.class);
+         printStats(result);
+      }, reply -> {
+         String errMsg = (String) ManagementHelper.getResult(reply, String.class);
+         getActionContext().err.println("Failed to get Stats for Queues. Reason: " + errMsg);
       });
    }
 
@@ -187,20 +278,45 @@ public class StatQueue extends AbstractAction {
       JsonArray array = queuesAsJsonObject.getJsonArray("data");
 
       int[] columnSizes = new int[FIELD.values().length];
+      boolean[] centralize = new boolean[columnSizes.length];
+
+      ArrayList<String>[] fieldTitles = new ArrayList[columnSizes.length];
 
       FIELD[] fields = FIELD.values();
       for (int i = 0; i < fields.length; i++) {
-         columnSizes[i] = fields[i].toString().length();
+         if (singleLineHeader) {
+            columnSizes[i] = fields[i].toString().length();
+         } else {
+            ArrayList<String> splitTitleArrayList = new ArrayList<>();
+            String[] splitTitleStringArray = fields[i].toString().split("_");
+            centralize[i] = fields[i].center;
+
+            for (String s : splitTitleStringArray) {
+               splitTitleArrayList.add(s);
+               columnSizes[i] = Math.max(columnSizes[i], s.length());
+            }
+
+            fieldTitles[i] = splitTitleArrayList;
+         }
       }
 
       for (int i = 0; i < array.size(); i++) {
          getColumnSizes(array.getJsonObject(i), columnSizes);
       }
 
-      printHeadings(columnSizes);
+      TableOut tableOut = new TableOut("|", 2, columnSizes);
+
+      if (singleLineHeader) {
+         printHeadings(columnSizes);
+      } else {
+         tableOut.print(getActionContext().out, fieldTitles, centralize);
+      }
 
       for (int i = 0; i < array.size(); i++) {
-         printQueueStats(array.getJsonObject(i), columnSizes);
+         if (!includeManagement && array.getJsonObject(i).getString("name").contains(MANAGEMENT_QUEUE)) {
+            continue;
+         }
+         printQueueStats(array.getJsonObject(i), columnSizes, centralize, tableOut);
          statCount++;
       }
 
@@ -211,9 +327,18 @@ public class StatQueue extends AbstractAction {
 
    private void getColumnSizes(JsonObject jsonObject, int[] columnSizes) {
       int i = 0;
+      if (!includeManagement && jsonObject.getString("name").startsWith(MANAGEMENT_QUEUE)) {
+         return;
+      }
       for (FIELD e: FIELD.values()) {
-         if (jsonObject.getString(e.jsonId).length() > columnSizes[i]) {
-            columnSizes[i] = jsonObject.getString(e.jsonId).length();
+         if (jsonObject.containsKey(e.jsonId)) {
+            if (jsonObject.getString(e.jsonId).length() > columnSizes[i]) {
+               columnSizes[i] = jsonObject.getString(e.jsonId).length();
+            }
+         } else {
+            if (NOT_APPLICABLE.length() > columnSizes[i]) {
+               columnSizes[i] = NOT_APPLICABLE.length();
+            }
          }
          // enforce max
          if (columnSizes[i] > maxColumnSize && maxColumnSize != -1) {
@@ -235,7 +360,8 @@ public class StatQueue extends AbstractAction {
       getActionContext().out.println(stringBuilder);
    }
 
-   private void printQueueStats(JsonObject jsonObject, int[] columnSizes) {
+
+   private void printQueueStats(JsonObject jsonObject, int[] columnSizes, boolean[] center, TableOut tableOut) {
 
       //should not happen but just in case..
       if (jsonObject == null) {
@@ -245,15 +371,16 @@ public class StatQueue extends AbstractAction {
          return;
       }
 
-      // add 10 for the various '|' characters
-      StringBuilder stringBuilder = new StringBuilder(Arrays.stream(columnSizes).sum() + FIELD.values().length + 1).append('|');
-
       int i = 0;
+      String[] columns = new String[columnSizes.length];
       for (FIELD e: FIELD.values()) {
-         stringBuilder.append(paddingString(new StringBuilder(jsonObject.getString(e.jsonId)), columnSizes[i++])).append('|');
+         if (!jsonObject.containsKey(e.jsonId)) {
+            columns[i++] = NOT_APPLICABLE;
+         } else {
+            columns[i++] = jsonObject.getString(e.jsonId);
+         }
       }
-
-      getActionContext().out.println(stringBuilder);
+      tableOut.print(getActionContext().out, columns, center);
    }
 
    private StringBuilder paddingString(StringBuilder value, int maxColumnSize) {
