@@ -27,8 +27,6 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 
-import com.github.rvesse.airline.annotations.Command;
-import com.github.rvesse.airline.annotations.Option;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffers;
 import org.apache.activemq.artemis.api.core.SimpleString;
@@ -63,25 +61,25 @@ import org.apache.activemq.artemis.utils.ExecutorFactory;
 import org.apache.activemq.artemis.utils.actors.ArtemisExecutor;
 import org.apache.activemq.artemis.utils.collections.LinkedList;
 import org.apache.activemq.artemis.utils.collections.LinkedListIterator;
+import picocli.CommandLine.Command;
+import picocli.CommandLine.Option;
 
 @Command(name = "print", description = "Print data records information. WARNING: don't use while a production server is running.")
 public class PrintData extends DBOption {
 
-
-   @Option(name = "--safe", description = "Print your data structure without showing your data.")
+   @Option(names = "--safe", description = "Print your data structure without showing your data.")
    private boolean safe = false;
 
-
-   @Option(name = "--reclaimed", description = "Try to print as many records as possible from reclaimed files.")
+   @Option(names = "--reclaimed", description = "Try to print as many records as possible from reclaimed files.")
    private boolean reclaimed = false;
 
-   @Option(name = "--max-pages", description = "Maximum number of pages to read. Default: unlimited (-1).")
+   @Option(names = "--max-pages", description = "Maximum number of pages to read. Default: unlimited (-1).")
    private int maxPages = -1;
 
-   @Option(name = "--skip-bindings", description = "Do not print data from the bindings journal.")
+   @Option(names = "--skip-bindings", description = "Do not print data from the bindings journal.")
    private boolean skipBindings = false;
 
-   @Option(name = "--skip-journal", description = "Do not print data from the messages journal.")
+   @Option(names = "--skip-journal", description = "Do not print data from the messages journal.")
    private boolean skipJournal = false;
 
    private static final String BINDINGS_BANNER = "B I N D I N G S  J O U R N A L";
@@ -104,6 +102,8 @@ public class PrintData extends DBOption {
          }
       } catch (Exception e) {
          treatError(e, "data", "print");
+      } finally {
+         done();
       }
       return null;
    }
@@ -116,13 +116,13 @@ public class PrintData extends DBOption {
 
       printBanner(out, BINDINGS_BANNER);
 
-      DescribeJournal.printSurvivingRecords(storageManager.getBindingsJournal(), out, safe);
+      DescribeJournal bindings = DescribeJournal.printSurvivingRecords(storageManager.getBindingsJournal(), out, safe);
 
       printBanner(out, MESSAGES_BANNER);
 
       DescribeJournal describeJournal = DescribeJournal.printSurvivingRecords(storageManager.getMessageJournal(), out, safe);
 
-      printPages(describeJournal, storageManager, pagingmanager, out, safe, maxPages, null);
+      printPages(describeJournal, storageManager, pagingmanager, out, safe, maxPages, bindings);
 
       cleanup();
 
@@ -331,7 +331,7 @@ public class PrintData extends DBOption {
                            out.print(" (ACK)");
                         }
 
-                        if (cursorACKs.getCompletePages(q[i]).contains(Long.valueOf(pgid))) {
+                        if (cursorACKs.getCompletePages(q[i]).contains(pgid)) {
                            acked = true;
                            out.print(" (PG-COMPLETE)");
                         }
@@ -398,8 +398,8 @@ public class PrintData extends DBOption {
             CursorAckRecordEncoding encoding = new CursorAckRecordEncoding();
             encoding.decode(buff);
 
-            Long queueID = Long.valueOf(encoding.queueID);
-            Long pageNR = Long.valueOf(encoding.position.getPageNr());
+            Long queueID = encoding.queueID;
+            Long pageNR = encoding.position.getPageNr();
 
             if (!cursorInfo.getCompletePages(queueID).add(pageNR)) {
                System.err.println("Page " + pageNR + " has been already set as complete on queue " + queueID);
