@@ -16,17 +16,13 @@
  */
 package org.apache.activemq.artemis.cli.commands.tools.xml;
 
-import javax.xml.XMLConstants;
-import javax.xml.stream.XMLInputFactory;
 import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.transform.stax.StAXSource;
-import javax.xml.validation.Schema;
-import javax.xml.validation.SchemaFactory;
 import javax.xml.validation.Validator;
-import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStream;
+import java.lang.invoke.MethodHandles;
 import java.net.URL;
 import java.nio.ByteBuffer;
 import java.security.AccessController;
@@ -36,10 +32,9 @@ import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.TreeSet;
 
-import io.airlift.airline.Command;
-import io.airlift.airline.Option;
 import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.RoutingType;
@@ -54,18 +49,19 @@ import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
 import org.apache.activemq.artemis.api.core.client.ServerLocator;
 import org.apache.activemq.artemis.api.core.management.ManagementHelper;
 import org.apache.activemq.artemis.api.core.management.ResourceNames;
-import org.apache.activemq.artemis.cli.commands.ActionAbstract;
 import org.apache.activemq.artemis.cli.commands.ActionContext;
+import org.apache.activemq.artemis.cli.commands.messages.ConnectionConfigurationAbtract;
 import org.apache.activemq.artemis.core.filter.impl.FilterImpl;
 import org.apache.activemq.artemis.core.protocol.core.impl.PacketImpl;
 import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnectorFactory;
 import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;
-import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
 import org.apache.activemq.artemis.utils.ClassloadingUtil;
 import org.apache.activemq.artemis.utils.ListUtil;
+import org.apache.activemq.artemis.utils.XmlProvider;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import java.lang.invoke.MethodHandles;
+import picocli.CommandLine.Command;
+import picocli.CommandLine.Option;
 
 /**
  * Read XML output from <code>org.apache.activemq.artemis.core.persistence.impl.journal.XmlDataExporter</code>, create a core session, and
@@ -73,7 +69,7 @@ import java.lang.invoke.MethodHandles;
  * for speed and simplicity.
  */
 @Command(name = "imp", description = "Import all message-data using an XML that could be interpreted by any system.")
-public final class XmlDataImporter extends ActionAbstract {
+public final class XmlDataImporter extends ConnectionConfigurationAbtract {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
@@ -93,58 +89,42 @@ public final class XmlDataImporter extends ActionAbstract {
    HashMap<String, String> oldPrefixTranslation = new HashMap<>();
 
    private ClientSession session;
+   private ClientProducer producer;
 
-   @Option(name = "--host", description = "The host used to import the data (default localhost)")
-   public String host = "localhost";
+   @Deprecated(forRemoval = true)
+   @Option(names = "--host", description = "The host used to import the data. Default: null.", hidden = true)
+   public String host = null;
 
-   @Option(name = "--port", description = "The port used to import the data (default 61616)")
+   @Deprecated(forRemoval = true)
+   @Option(names = "--port", description = "The port used to import the data. Default: 61616.", hidden = true)
    public int port = 61616;
 
-   @Option(name = "--transaction", description = "If this is set to true you will need a whole transaction to commit at the end. (default false)")
+   @Option(names = "--transaction", description = "Import every message using a single transction. If anything goes wrong during the process the entire import will be aborted. Default: false.", hidden = true)
    public boolean transactional;
 
-   @Option(name = "--user", description = "User name used to import the data. (default null)")
-   public String user = null;
+   @Option(names = "--commit-interval", description = "How often to commit.", hidden = true)
+   public int commitInterval = 1000;
 
-   @Option(name = "--password", description = "User name used to import the data. (default null)")
-   public String password = null;
-
-   @Option(name = "--input", description = "The input file name (default=exp.dmp)", required = true)
+   @Option(names = "--input", description = "The input file name. Default: exp.dmp.", required = true)
    public String input = "exp.dmp";
 
-   @Option(name = "--sort", description = "Sort the messages from the input (used for older versions that won't sort messages)")
+   @Option(names = "--sort", description = "Sort the messages from the input (used for older versions that won't sort messages).")
    public boolean sort = false;
 
-   @Option(name = "--legacy-prefixes", description = "Do not remove prefixes from legacy imports")
+   @Option(names = "--legacy-prefixes", description = "Do not remove prefixes from legacy imports.")
    public boolean legacyPrefixes = false;
 
    TreeSet<XMLMessageImporter.MessageInfo> messages;
 
-   public String getPassword() {
-      return password;
-   }
-
-   public void setPassword(String password) {
-      this.password = password;
-   }
-
-   public String getUser() {
-      return user;
-   }
-
-   public void setUser(String user) {
-      this.user = user;
-   }
-
    @Override
    public Object execute(ActionContext context) throws Exception {
-      process(input, host, port, transactional);
+      process(input, host, port);
       return null;
    }
 
-   public void process(String inputFileName, String host, int port, boolean transactional) throws Exception {
+   public void process(String inputFileName, String host, int port) throws Exception {
       try (FileInputStream inputFile = new FileInputStream(inputFileName)) {
-         this.process(inputFile, host, port, transactional);
+         this.process(inputFile, host, port);
       }
    }
 
@@ -175,11 +155,13 @@ public final class XmlDataImporter extends ActionAbstract {
    public void process(InputStream inputStream,
                        ClientSession session,
                        ClientSession managementSession) throws Exception {
-      reader = XMLInputFactory.newInstance().createXMLStreamReader(inputStream);
+      Objects.requireNonNull(inputStream);
+      reader = XmlProvider.createXMLStreamReader(inputStream);
       messageReader = new XMLMessageImporter(reader, session);
       messageReader.setOldPrefixTranslation(oldPrefixTranslation);
 
       this.session = session;
+      this.producer = session.createProducer();
       if (managementSession != null) {
          this.managementSession = managementSession;
       } else {
@@ -187,29 +169,52 @@ public final class XmlDataImporter extends ActionAbstract {
       }
 
       processXml();
-
    }
 
-   public void process(InputStream inputStream, String host, int port, boolean transactional) throws Exception {
-      HashMap<String, Object> connectionParams = new HashMap<>();
-      connectionParams.put(TransportConstants.HOST_PROP_NAME, host);
-      connectionParams.put(TransportConstants.PORT_PROP_NAME, Integer.toString(port));
-      ServerLocator serverLocator = ActiveMQClient.createServerLocatorWithoutHA(new TransportConfiguration(NettyConnectorFactory.class.getName(), connectionParams));
+   public void process(InputStream inputStream, String host, int port) throws Exception {
+      ServerLocator serverLocator;
+      if (host != null) {
+         Map<String, Object> connectionParams = Map.of(
+            TransportConstants.HOST_PROP_NAME, host,
+            TransportConstants.PORT_PROP_NAME, Integer.toString(port)
+         );
+         serverLocator = ActiveMQClient.createServerLocatorWithoutHA(new TransportConfiguration(NettyConnectorFactory.class.getName(), connectionParams));
+      } else {
+         serverLocator = ActiveMQClient.createServerLocator(brokerURL);
+      }
       ClientSessionFactory sf = serverLocator.createSessionFactory();
 
-      ClientSession session;
-      ClientSession managementSession;
+      ClientSession session = null;
+      ClientSession managementSession = null;
+
+      try {
+
+         if (user != null || password != null) {
+            session = sf.createSession(user, password, false, false, true, false, 0);
+            managementSession = sf.createSession(user, password, false, true, true, false, 0);
+         } else {
+            session = sf.createSession(false, false, true);
+            managementSession = sf.createSession(false, true, true);
+         }
+         localSession = true;
+
+         producer = session.createProducer();
+
+         process(inputStream, session, managementSession);
+      } finally {
+         try {
+            session.commit();
+            session.close();
+         } catch (Throwable ignored) {
+         }
+         try {
+            managementSession.commit();
+            managementSession.close();
+         } catch (Throwable ignored) {
+         }
+
 
-      if (user != null || password != null) {
-         session = sf.createSession(user, password, false, !transactional, true, false, 0);
-         managementSession = sf.createSession(user, password, false, true, true, false, 0);
-      } else {
-         session = sf.createSession(false, !transactional, true);
-         managementSession = sf.createSession(false, true, true);
       }
-      localSession = true;
-
-      process(inputStream, session, managementSession);
    }
 
    public void validate(String fileName) throws Exception {
@@ -219,11 +224,8 @@ public final class XmlDataImporter extends ActionAbstract {
    }
 
    public void validate(InputStream inputStream) throws Exception {
-      XMLStreamReader reader = XMLInputFactory.newInstance().createXMLStreamReader(inputStream);
-      SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
-      Schema schema = factory.newSchema(XmlDataImporter.findResource("schema/artemis-import-export.xsd"));
-
-      Validator validator = schema.newValidator();
+      XMLStreamReader reader = XmlProvider.createXMLStreamReader(inputStream);
+      Validator validator = XmlProvider.newValidator(XmlDataImporter.findResource("schema/artemis-import-export.xsd"));
       validator.validate(new StAXSource(reader));
       reader.close();
    }
@@ -246,54 +248,57 @@ public final class XmlDataImporter extends ActionAbstract {
             }
          });
       }
-      try {
-         while (reader.hasNext()) {
-            if (logger.isDebugEnabled()) {
-               logger.debug("EVENT:[{}][{}] ", reader.getLocation().getLineNumber(), reader.getLocation().getColumnNumber());
-            }
-
-            if (reader.getEventType() == XMLStreamConstants.START_ELEMENT) {
-               if (XmlDataConstants.OLD_BINDING.equals(reader.getLocalName())) {
-                  oldBinding(); // export from 1.x
-               } else if (XmlDataConstants.QUEUE_BINDINGS_CHILD.equals(reader.getLocalName())) {
-                  bindQueue();
-               } else if (XmlDataConstants.ADDRESS_BINDINGS_CHILD.equals(reader.getLocalName())) {
-                  bindAddress();
-               } else if (XmlDataConstants.MESSAGES_CHILD.equals(reader.getLocalName())) {
-                  processMessage();
-               }
-            }
-            reader.next();
+      while (reader.hasNext()) {
+         if (logger.isDebugEnabled()) {
+            logger.debug("EVENT:[{}][{}] ", reader.getLocation().getLineNumber(), reader.getLocation().getColumnNumber());
          }
 
-         if (sort) {
-            for (XMLMessageImporter.MessageInfo msgtmp : messages) {
-               sendMessage(msgtmp.queues, msgtmp.message, msgtmp.tempFile);
+         if (reader.getEventType() == XMLStreamConstants.START_ELEMENT) {
+            if (XmlDataConstants.OLD_BINDING.equals(reader.getLocalName())) {
+               oldBinding(); // export from 1.x
+            } else if (XmlDataConstants.QUEUE_BINDINGS_CHILD.equals(reader.getLocalName())) {
+               bindQueue();
+            } else if (XmlDataConstants.ADDRESS_BINDINGS_CHILD.equals(reader.getLocalName())) {
+               bindAddress();
+            } else if (XmlDataConstants.MESSAGES_CHILD.equals(reader.getLocalName())) {
+               processMessage();
             }
          }
+         reader.next();
+      }
 
-         if (!session.isAutoCommitSends()) {
-            session.commit();
-         }
-      } finally {
-         // if the session was created in our constructor then close it (otherwise the caller will close it)
-         if (localSession) {
-            session.close();
-            managementSession.close();
+      if (sort) {
+         long messageNr = 0;
+         for (XMLMessageImporter.MessageInfo msgtmp : messages) {
+            sendMessage(msgtmp.queues, msgtmp.message);
+            messageNr++;
+            if (messageNr % commitInterval == 0) {
+               session.commit();
+            }
          }
       }
+
+      session.commit();
    }
 
+   long messageNr = 0;
+
    private void processMessage() throws Exception {
+      messageNr++;
+
+      if (messageNr % commitInterval == 0) {
+         getActionContext().err.println("Processed " + messageNr + " messages");
+         session.commit();
+      }
       XMLMessageImporter.MessageInfo info = messageReader.readMessage(false);
       if (sort) {
          messages.add(info);
       } else {
-         sendMessage(info.queues, info.message, info.tempFile);
+         sendMessage(info.queues, info.message);
       }
    }
 
-   private void sendMessage(List<String> queues, Message message, File tempFileName) throws Exception {
+   private void sendMessage(List<String> queues, Message message) throws Exception {
       final String destination = addressMap.get(queues.get(0));
       final ByteBuffer buffer = ByteBuffer.allocate(queues.size() * 8);
 
@@ -304,7 +309,7 @@ public final class XmlDataImporter extends ActionAbstract {
       }
 
       for (String queue : queues) {
-         long queueID;
+         long queueID = -1;
 
          if (queueIDs.containsKey(queue)) {
             queueID = queueIDs.get(queue);
@@ -320,17 +325,27 @@ public final class XmlDataImporter extends ActionAbstract {
                   logger.debug("Requesting ID for: {}", queue);
                }
                ClientMessage reply = requestor.request(managementMessage);
-               Number idObject = (Number) ManagementHelper.getResult(reply);
-               queueID = idObject.longValue();
+               if (ManagementHelper.hasOperationSucceeded(reply)) {
+                  Number idObject = (Number) ManagementHelper.getResult(reply);
+                  queueID = idObject.longValue();
+               } else {
+                  if (debugLog) {
+                     logger.debug("Failed to get ID for {}, reply: {}", queue, ManagementHelper.getResult(reply, String.class));
+                  }
+               }
             }
 
             if (debugLog) {
                logger.debug("ID for {} is: {}", queue, queueID);
             }
-            queueIDs.put(queue, queueID);  // store it so we don't have to look it up every time
+            if (queueID != -1) {
+               queueIDs.put(queue, queueID);  // store it so we don't have to look it up every time
+            }
          }
 
-         buffer.putLong(queueID);
+         if (queueID != -1) {
+            buffer.putLong(queueID);
+         }
          if (debugLog) {
             debugLogMessage.append(queue).append(", ");
          }
@@ -342,21 +357,7 @@ public final class XmlDataImporter extends ActionAbstract {
       }
 
       message.putBytesProperty(Message.HDR_ROUTE_TO_IDS, buffer.array());
-      try (ClientProducer producer = session.createProducer(destination)) {
-         producer.send(message);
-      }
-
-      if (tempFileName != null) {
-         try {
-            // this is to make sure the large message is sent before we delete it
-            // to avoid races
-            session.commit();
-         } catch (Throwable dontcare) {
-         }
-         if (!tempFileName.delete()) {
-            ActiveMQServerLogger.LOGGER.couldNotDeleteTempFile(tempFileName.getAbsolutePath());
-         }
-      }
+      producer.send(SimpleString.toSimpleString(destination), message);
    }
 
    private void oldBinding() throws Exception {
