@@ -49,13 +49,15 @@ import org.apache.activemq.artemis.jms.client.ActiveMQXAQueueConnectionFactory;
 import org.apache.activemq.artemis.jms.client.ActiveMQXATopicConnectionFactory;
 import org.apache.activemq.artemis.utils.RandomUtil;
 import org.apache.commons.beanutils.BeanUtilsBean;
-import org.jboss.logging.Logger;
 import org.junit.Assert;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
 
 public class ConnectionFactoryURITest {
 
-   private static final Logger log = Logger.getLogger(ConnectionFactoryURITest.class);
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
    ConnectionFactoryParser parser = new ConnectionFactoryParser();
 
@@ -171,13 +173,38 @@ public class ConnectionFactoryURITest {
 
    @Test
    public void testTCPAllProperties() throws Exception {
-      StringBuilder sb = new StringBuilder();
-      sb.append("tcp://localhost:3030?ha=true");
-      BeanUtilsBean bean = new BeanUtilsBean();
-      ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
-      populate(sb, bean, factory);
-      ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
-      checkEquals(bean, factory, factory2);
+      ignoreList.add("deserializationBlackList");
+      ignoreList.add("deserializationWhiteList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("tcp://localhost:3030?ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationBlackList");
+         ignoreList.remove("deserializationWhiteList");
+      }
+   }
+
+   @Test
+   public void testTCPAllPropertiesWithDeprecatedProps() throws Exception {
+      ignoreList.add("deserializationDenyList");
+      ignoreList.add("deserializationAllowList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("tcp://localhost:3030?ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationDenyList");
+         ignoreList.remove("deserializationAllowList");
+      }
    }
 
    @Test
@@ -293,13 +320,38 @@ public class ConnectionFactoryURITest {
 
    @Test
    public void testUDPAllProperties() throws Exception {
-      StringBuilder sb = new StringBuilder();
-      sb.append("udp://localhost:3030?ha=true");
-      BeanUtilsBean bean = new BeanUtilsBean();
-      ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
-      populate(sb, bean, factory);
-      ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
-      checkEquals(bean, factory, factory2);
+      ignoreList.add("deserializationBlackList");
+      ignoreList.add("deserializationWhiteList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("udp://localhost:3030?ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationBlackList");
+         ignoreList.remove("deserializationWhiteList");
+      }
+   }
+
+   @Test
+   public void testUDPAllPropertiesWithDeprecatedListProps() throws Exception {
+      ignoreList.add("deserializationDenyList");
+      ignoreList.add("deserializationAllowList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("udp://localhost:3030?ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationDenyList");
+         ignoreList.remove("deserializationAllowList");
+      }
    }
 
    @Test
@@ -354,13 +406,38 @@ public class ConnectionFactoryURITest {
 
    @Test
    public void testJGroupsAllProperties() throws Exception {
-      StringBuilder sb = new StringBuilder();
-      sb.append("jgroups://?file=param=value;param=value&channelName=channelName&ha=true");
-      BeanUtilsBean bean = new BeanUtilsBean();
-      ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
-      populate(sb, bean, factory);
-      ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
-      checkEquals(bean, factory, factory2);
+      ignoreList.add("deserializationBlackList");
+      ignoreList.add("deserializationWhiteList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("jgroups://?file=param=value;param=value&channelName=channelName&ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationBlackList");
+         ignoreList.remove("deserializationWhiteList");
+      }
+   }
+
+   @Test
+   public void testJGroupsAllPropertiesWithDeprecatedListProps() throws Exception {
+      ignoreList.add("deserializationDenyList");
+      ignoreList.add("deserializationAllowList");
+      try {
+         StringBuilder sb = new StringBuilder();
+         sb.append("jgroups://?file=param=value;param=value&channelName=channelName&ha=true");
+         BeanUtilsBean bean = new BeanUtilsBean();
+         ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(true, (TransportConfiguration) null);
+         populate(sb, bean, factory);
+         ActiveMQConnectionFactory factory2 = parser.newObject(new URI(sb.toString()), null);
+         checkEquals(bean, factory, factory2);
+      } finally {
+         ignoreList.remove("deserializationDenyList");
+         ignoreList.remove("deserializationAllowList");
+      }
    }
 
    @Test
@@ -407,7 +484,7 @@ public class ConnectionFactoryURITest {
          if (ignoreList.contains(descriptor.getName())) {
             continue;
          }
-         log.info("name::" + descriptor.getName());
+         logger.info("name::{}", descriptor.getName());
          if (descriptor.getWriteMethod() != null && descriptor.getReadMethod() != null) {
             if (descriptor.getPropertyType() == String.class) {
                String value = RandomUtil.randomString();
