@@ -35,19 +35,20 @@ import io.netty.buffer.ByteBuf;
 import io.netty.channel.EventLoop;
 import org.apache.activemq.artemis.api.core.ActiveMQSecurityException;
 import org.apache.activemq.artemis.api.core.SimpleString;
-import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnection;
 import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;
 import org.apache.activemq.artemis.core.security.CheckType;
 import org.apache.activemq.artemis.core.security.SecurityAuth;
 import org.apache.activemq.artemis.protocol.amqp.broker.AMQPConnectionCallback;
 import org.apache.activemq.artemis.protocol.amqp.broker.AMQPSessionCallback;
 import org.apache.activemq.artemis.protocol.amqp.broker.ProtonProtocolManager;
+import org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationAddressSenderController;
+import org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationQueueSenderController;
 import org.apache.activemq.artemis.protocol.amqp.connect.mirror.AMQPMirrorControllerSource;
 import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPException;
+import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPSecurityException;
 import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolLogger;
 import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolMessageBundle;
 import org.apache.activemq.artemis.protocol.amqp.proton.handler.EventHandler;
-import org.apache.activemq.artemis.protocol.amqp.proton.handler.ExecutorNettyAdapter;
 import org.apache.activemq.artemis.protocol.amqp.proton.handler.ExtCapability;
 import org.apache.activemq.artemis.protocol.amqp.proton.handler.ProtonHandler;
 import org.apache.activemq.artemis.protocol.amqp.sasl.AnonymousServerSASL;
@@ -76,16 +77,36 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import java.lang.invoke.MethodHandles;
 
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_ADDRESS_RECEIVER;
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_CONTROL_LINK;
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_BASE_VALIDATION_ADDRESS;
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_EVENT_LINK;
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_QUEUE_RECEIVER;
+import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.AMQP_LINK_INITIALIZER_KEY;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.FAILOVER_SERVER_LIST;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.HOSTNAME;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.NETWORK_HOST;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.PORT;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.SCHEME;
+import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.verifyDesiredCapability;;
 
 public class AMQPConnectionContext extends ProtonInitializable implements EventHandler {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
+   public void disableAutoRead() {
+      handler.requireHandler();
+      connectionCallback.getTransportConnection().setAutoRead(false);
+      handler.setReadable(false);
+   }
+
+   public void enableAutoRead() {
+      handler.requireHandler();
+      connectionCallback.getTransportConnection().setAutoRead(true);
+      getHandler().setReadable(true);
+      flush();
+   }
+
    public static final Symbol CONNECTION_OPEN_FAILED = Symbol.valueOf("amqp:connection-establishment-failed");
    public static final String AMQP_CONTAINER_ID = "amqp-container-id";
    private static final FutureTask<Void> VOID_FUTURE = new FutureTask<>(() -> { }, null);
@@ -98,8 +119,7 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
    private final ClientSASLFactory saslClientFactory;
    private final Map<Symbol, Object> connectionProperties = new HashMap<>();
    private final ScheduledExecutorService scheduledPool;
-
-   private LinkCloseListener linkCloseListener;
+   private final Map<String, LinkCloseListener> linkCloseListeners = new ConcurrentHashMap<>();
 
    private final Map<Session, AMQPSessionContext> sessions = new ConcurrentHashMap<>();
 
@@ -111,7 +131,7 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
    private final boolean bridgeConnection;
 
    private final ScheduleOperator scheduleOp = new ScheduleOperator(new ScheduleRunnable());
-   private final AtomicReference<Future<?>> scheduledFutureRef = new AtomicReference(VOID_FUTURE);
+   private final AtomicReference<Future<?>> scheduledFutureRef = new AtomicReference<>(VOID_FUTURE);
 
    private String user;
    private String password;
@@ -160,12 +180,7 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
 
       this.scheduledPool = scheduledPool;
       connectionCallback.setConnection(this);
-      EventLoop nettyExecutor;
-      if (connectionCallback.getTransportConnection() instanceof NettyConnection) {
-         nettyExecutor = ((NettyConnection) connectionCallback.getTransportConnection()).getNettyChannel().eventLoop();
-      } else {
-         nettyExecutor = new ExecutorNettyAdapter(protocolManager.getServer().getExecutorFactory().getExecutor());
-      }
+      EventLoop nettyExecutor = connectionCallback.getTransportConnection().getEventLoop();
       this.handler = new ProtonHandler(nettyExecutor, protocolManager.getServer().getExecutorFactory().getExecutor(), isIncomingConnection && saslClientFactory == null);
       handler.addEventHandler(this);
       Transport transport = handler.getTransport();
@@ -182,15 +197,50 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
       }
    }
 
-   public LinkCloseListener getLinkCloseListener() {
-      return linkCloseListener;
+   public boolean isLargeMessageSync() {
+      return connectionCallback.isLargeMessageSync();
    }
 
-   public AMQPConnectionContext setLinkCloseListener(LinkCloseListener linkCloseListener) {
-      this.linkCloseListener = linkCloseListener;
+   @Override
+   public void initialize() throws Exception {
+      initialized = true;
+   }
+
+   /**
+    * Adds a listener that will be invoked any time an AMQP link is remotely closed
+    * before having been closed on this end of the connection.
+    *
+    * @param id
+    *    A unique ID assigned to the listener used to later remove it if needed.
+    * @param linkCloseListener
+    *    The instance of a closed listener.
+    *
+    * @return this connection context instance.
+    */
+   public AMQPConnectionContext addLinkRemoteCloseListener(String id, LinkCloseListener linkCloseListener) {
+      linkCloseListeners.put(id, linkCloseListener);
       return this;
    }
 
+   /**
+    * Remove the link remote close listener that is identified by the given ID.
+    *
+    * @param id
+    *    The unique ID assigned to the listener when it was added.
+    */
+   public void removeLinkRemoteCloseListener(String id) {
+      linkCloseListeners.remove(id);
+   }
+
+   /**
+    * Clear all link remote close listeners, usually done before connection
+    * termination to avoid any remote close events triggering processing
+    * after the connection shutdown has already started.
+    */
+   public void clearLinkRemoteCloseListeners() {
+      linkCloseListeners.clear();
+   }
+
    public boolean isBridgeConnection() {
       return bridgeConnection;
    }
@@ -342,12 +392,11 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
    }
 
    protected void remoteLinkOpened(Link link) throws Exception {
+      final AMQPSessionContext protonSession = getSessionExtension(link.getSession());
 
-      AMQPSessionContext protonSession = getSessionExtension(link.getSession());
-
-      Runnable runnable = link.attachments().get(Runnable.class, Runnable.class);
+      final Runnable runnable = link.attachments().get(AMQP_LINK_INITIALIZER_KEY, Runnable.class);
       if (runnable != null) {
-         link.attachments().set(Runnable.class, Runnable.class, null);
+         link.attachments().set(AMQP_LINK_INITIALIZER_KEY, Runnable.class, null);
          runnable.run();
          return;
       }
@@ -358,71 +407,113 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
 
       link.setSource(link.getRemoteSource());
       link.setTarget(link.getRemoteTarget());
+
       if (link instanceof Receiver) {
          Receiver receiver = (Receiver) link;
          if (link.getRemoteTarget() instanceof Coordinator) {
             Coordinator coordinator = (Coordinator) link.getRemoteTarget();
             protonSession.addTransactionHandler(coordinator, receiver);
+         } else if (isReplicaTarget(receiver)) {
+            handleReplicaTargetLinkOpened(protonSession, receiver);
+         } else if (isFederationControlLink(receiver)) {
+            handleFederationControlLinkOpened(protonSession, receiver);
+         } else if (isFederationEventLink(receiver)) {
+            protonSession.addFederationEventProcessor(receiver);
          } else {
-            if (isReplicaTarget(receiver)) {
-               try {
-                  try {
-                     protonSession.getSessionSPI().check(SimpleString.toSimpleString(link.getTarget().getAddress()), CheckType.SEND, getSecurityAuth());
-                  } catch (ActiveMQSecurityException e) {
-                     throw ActiveMQAMQPProtocolMessageBundle.BUNDLE.securityErrorCreatingProducer(e.getMessage());
-                  }
-
-                  if (!verifyDesiredCapabilities(receiver, AMQPMirrorControllerSource.MIRROR_CAPABILITY)) {
-                     throw ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingDesiredCapability(AMQPMirrorControllerSource.MIRROR_CAPABILITY.toString());
-                  }
-               } catch (ActiveMQAMQPException e) {
-                  logger.warn(e.getMessage(), e);
-
-                  link.setTarget(null);
-                  link.setCondition(new ErrorCondition(e.getAmqpError(), e.getMessage()));
-                  link.close();
-
-                  return;
-               }
-
-               receiver.setOfferedCapabilities(new Symbol[]{AMQPMirrorControllerSource.MIRROR_CAPABILITY});
-               protonSession.addReplicaTarget(receiver);
-            } else {
-               protonSession.addReceiver(receiver);
-            }
+            protonSession.addReceiver(receiver);
          }
       } else {
-         Sender sender = (Sender) link;
-         protonSession.addSender(sender);
-      }
-   }
-
-
-   protected boolean verifyDesiredCapabilities(Receiver reciever, Symbol s) {
-
-      if (reciever.getRemoteDesiredCapabilities() == null) {
-         return false;
-      }
-
-      boolean foundS = false;
-      for (Symbol b : reciever.getRemoteDesiredCapabilities()) {
-         if (b.equals(s)) {
-            foundS = true;
-            break;
+         final Sender sender = (Sender) link;
+         if (isFederationAddressReceiver(sender)) {
+            protonSession.addSender(sender, new AMQPFederationAddressSenderController(protonSession));
+         } else if (isFederationQueueReceiver(sender)) {
+            protonSession.addSender(sender, new AMQPFederationQueueSenderController(protonSession));
+         } else if (isFederationEventLink(sender)) {
+            protonSession.addFederationEventDispatcher(sender);
+         } else {
+            protonSession.addSender(sender);
          }
       }
-      if (!foundS) {
-         return false;
-      }
-
-      return true;
    }
 
+   private void handleReplicaTargetLinkOpened(AMQPSessionContext protonSession, Receiver receiver) throws Exception {
+      try {
+         try {
+            protonSession.getSessionSPI().check(SimpleString.toSimpleString(receiver.getTarget().getAddress()), CheckType.SEND, getSecurityAuth());
+         } catch (ActiveMQSecurityException e) {
+            throw ActiveMQAMQPProtocolMessageBundle.BUNDLE.securityErrorCreatingProducer(e.getMessage());
+         }
 
-   private boolean isReplicaTarget(Link link) {
+         if (!verifyDesiredCapability(receiver, AMQPMirrorControllerSource.MIRROR_CAPABILITY)) {
+            throw ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingDesiredCapability(AMQPMirrorControllerSource.MIRROR_CAPABILITY.toString());
+         }
+      } catch (ActiveMQAMQPException e) {
+         logger.warn(e.getMessage(), e);
+
+         receiver.setTarget(null);
+         receiver.setCondition(new ErrorCondition(e.getAmqpError(), e.getMessage()));
+         receiver.close();
+
+         return;
+      }
+
+      // We need to check if the remote desires to send us tunneled core messages or not, and if
+      // we support that we need to offer that back so it knows it can actually do core tunneling.
+      if (verifyDesiredCapability(receiver, AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT)) {
+         receiver.setOfferedCapabilities(new Symbol[] {AMQPMirrorControllerSource.MIRROR_CAPABILITY,
+                                                       AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT});
+      } else {
+         receiver.setOfferedCapabilities(new Symbol[]{AMQPMirrorControllerSource.MIRROR_CAPABILITY});
+      }
+
+      protonSession.addReplicaTarget(receiver);
+   }
+
+   private void handleFederationControlLinkOpened(AMQPSessionContext protonSession, Receiver receiver) throws Exception {
+      try {
+         try {
+            protonSession.getSessionSPI().check(SimpleString.toSimpleString(FEDERATION_BASE_VALIDATION_ADDRESS), CheckType.SEND, getSecurityAuth());
+         } catch (ActiveMQSecurityException e) {
+            throw new ActiveMQAMQPSecurityException(
+               "User does not have permission to attach to the federation control address");
+         }
+
+         protonSession.addFederationCommandProcessor(receiver);
+      } catch (ActiveMQAMQPException e) {
+         logger.warn(e.getMessage(), e);
+
+         receiver.setTarget(null);
+         receiver.setCondition(new ErrorCondition(e.getAmqpError(), e.getMessage()));
+         receiver.close();
+
+         return;
+      }
+   }
+
+   private static boolean isReplicaTarget(Link link) {
       return link != null && link.getTarget() != null && link.getTarget().getAddress() != null && link.getTarget().getAddress().startsWith(ProtonProtocolManager.MIRROR_ADDRESS);
    }
 
+   private static boolean isFederationControlLink(Receiver receiver) {
+      return verifyDesiredCapability(receiver, FEDERATION_CONTROL_LINK);
+   }
+
+   private static boolean isFederationEventLink(Sender sender) {
+      return verifyDesiredCapability(sender, FEDERATION_EVENT_LINK);
+   }
+
+   private static boolean isFederationEventLink(Receiver receiver) {
+      return verifyDesiredCapability(receiver, FEDERATION_EVENT_LINK);
+   }
+
+   private static boolean isFederationQueueReceiver(Sender sender) {
+      return verifyDesiredCapability(sender, FEDERATION_QUEUE_RECEIVER);
+   }
+
+   private static boolean isFederationAddressReceiver(Sender sender) {
+      return verifyDesiredCapability(sender, FEDERATION_ADDRESS_RECEIVER);
+   }
+
    public Symbol[] getConnectionCapabilitiesOffered() {
       URI tc = connectionCallback.getFailoverList();
       if (tc != null) {
@@ -594,7 +685,6 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
          try {
             validatedUser = protocolManager.getServer().validateUser(user, password, connectionCallback.getProtonConnectionDelegate(), protocolManager.getSecurityDomain());
          } catch (ActiveMQSecurityException e) {
-            logger.warn(e.getMessage(), e);
             ErrorCondition error = new ErrorCondition();
             error.setCondition(AmqpError.UNAUTHORIZED_ACCESS);
             error.setDescription(e.getMessage() == null ? e.getClass().getSimpleName() : e.getMessage());
@@ -687,9 +777,13 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
 
    @Override
    public void onRemoteOpen(Session session) throws Exception {
-      handler.requireHandler();
-      getSessionExtension(session).initialize();
-      session.open();
+      // If connection already closed then we shouldn't react to the most likely
+      // pipelined Begin event.
+      if (session.getConnection().getLocalState() != EndpointState.CLOSED) {
+         handler.requireHandler();
+         getSessionExtension(session).initialize();
+         session.open();
+      }
    }
 
    @Override
@@ -709,7 +803,11 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
 
    @Override
    public void onRemoteOpen(Link link) throws Exception {
-      remoteLinkOpened(link);
+      // If connection already closed then we shouldn't react to the most likely
+      // pipelined Attach event.
+      if (link.getSession().getConnection().getLocalState() != EndpointState.CLOSED) {
+         remoteLinkOpened(link);
+      }
    }
 
    @Override
@@ -723,9 +821,15 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
    public void onRemoteClose(Link link) throws Exception {
       handler.requireHandler();
 
-      if (linkCloseListener != null) {
-         linkCloseListener.onClose(link);
-      }
+      final AtomicReference<Exception> handlerThrew = new AtomicReference<>();
+
+      linkCloseListeners.forEach((k, v) -> {
+         try {
+            v.onClose(link);
+         } catch (Exception e) {
+            handlerThrew.compareAndSet(null, e);
+         }
+      });
 
       ProtonDeliveryHandler linkContext = (ProtonDeliveryHandler) link.getContext();
       if (linkContext != null) {
@@ -736,14 +840,13 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
          }
       }
 
-      /// we have to perform the link.close after the linkContext.close is finished.
-      // linkeContext.close will perform a few executions on the netty loop,
-      // this has to come next
-      runLater(() -> {
-         link.close();
-         link.free();
-         flush();
-      });
+      link.close();
+      link.free();
+      flush();
+
+      if (handlerThrew.get() != null) {
+         throw handlerThrew.get();
+      }
    }
 
    @Override
@@ -780,7 +883,6 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
       }
    }
 
-
    private class LocalSecurity implements SecurityAuth {
       @Override
       public String getUsername() {
@@ -816,5 +918,4 @@ public class AMQPConnectionContext extends ProtonInitializable implements EventH
          return getProtocolManager().getSecurityDomain();
       }
    }
-
 }
