@@ -1054,6 +1054,21 @@ public class QueueControlImpl extends AbstractControl implements QueueControl {
       return value == null ? 0 : value;
    }
 
+   @Override
+   public boolean isInternalQueue() {
+      if (AuditLogger.isBaseLoggingEnabled()) {
+         AuditLogger.isInternal(queue);
+      }
+      checkStarted();
+
+      clearIO();
+      try {
+         return queue.isInternalQueue();
+      } finally {
+         blockOnIO();
+      }
+   }
+
    @Override
    public String countMessages(final String filterStr, final String groupByProperty) throws Exception {
       if (AuditLogger.isBaseLoggingEnabled()) {
@@ -1128,7 +1143,7 @@ public class QueueControlImpl extends AbstractControl implements QueueControl {
             Filter filter = FilterImpl.createFilter(filterStr);
             SimpleString groupByProperty = SimpleString.toSimpleString(groupByPropertyStr);
             if (filter == null && groupByProperty == null) {
-               result.put(null, Long.valueOf(getDeliveringCount()));
+               result.put(null, (long) getDeliveringCount());
             } else {
                Map<String, List<MessageReference>> deliveringMessages = queue.getDeliveringMessages();
                deliveringMessages.forEach((s, messageReferenceList) -> messageReferenceList.forEach(messageReference -> internalComputeMessage(result, filter, groupByProperty, messageReference.getMessage())));
@@ -1696,7 +1711,7 @@ public class QueueControlImpl extends AbstractControl implements QueueControl {
          try {
             long index = 0;
             long start = (long) (page - 1) * pageSize;
-            long end = Math.min(page * pageSize, queue.getMessageCount());
+            long end = Math.min((long) page * pageSize, queue.getMessageCount());
 
             ArrayList<CompositeData> c = new ArrayList<>();
             Filter thefilter = FilterImpl.createFilter(filter);
