@@ -17,14 +17,24 @@
 package org.apache.activemq.artemis.component;
 
 import javax.servlet.DispatcherType;
+import javax.servlet.ServletContextEvent;
+import javax.servlet.ServletContextListener;
+import javax.servlet.ServletRequestEvent;
+import javax.servlet.ServletRequestListener;
 import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.lang.invoke.MethodHandles;
 import java.net.URI;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.EnumSet;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.stream.Collectors;
 
 import org.apache.activemq.artemis.ActiveMQWebLogger;
@@ -34,12 +44,16 @@ import org.apache.activemq.artemis.dto.AppDTO;
 import org.apache.activemq.artemis.dto.BindingDTO;
 import org.apache.activemq.artemis.dto.ComponentDTO;
 import org.apache.activemq.artemis.dto.WebServerDTO;
+import org.apache.activemq.artemis.logs.AuditLogger;
 import org.apache.activemq.artemis.marker.WebServerComponentMarker;
+import org.apache.activemq.artemis.utils.ClassloadingUtil;
+import org.apache.activemq.artemis.utils.PemConfigUtil;
 import org.eclipse.jetty.security.DefaultAuthenticatorFactory;
 import org.eclipse.jetty.server.ConnectionFactory;
 import org.eclipse.jetty.server.CustomRequestLog;
 import org.eclipse.jetty.server.HttpConfiguration;
 import org.eclipse.jetty.server.HttpConnectionFactory;
+import org.eclipse.jetty.server.RequestLog;
 import org.eclipse.jetty.server.RequestLogWriter;
 import org.eclipse.jetty.server.SecureRequestCustomizer;
 import org.eclipse.jetty.server.Server;
@@ -47,14 +61,17 @@ import org.eclipse.jetty.server.ServerConnector;
 import org.eclipse.jetty.server.SslConnectionFactory;
 import org.eclipse.jetty.server.handler.ContextHandler;
 import org.eclipse.jetty.server.handler.HandlerList;
-import org.eclipse.jetty.server.handler.RequestLogHandler;
 import org.eclipse.jetty.server.handler.ResourceHandler;
 import org.eclipse.jetty.servlet.FilterHolder;
+import org.eclipse.jetty.util.Scanner;
 import org.eclipse.jetty.util.ssl.SslContextFactory;
+import org.eclipse.jetty.util.thread.QueuedThreadPool;
+import org.eclipse.jetty.util.thread.ScheduledExecutorScheduler;
 import org.eclipse.jetty.webapp.WebAppContext;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import java.lang.invoke.MethodHandles;
+
+import static org.apache.activemq.artemis.core.remoting.impl.ssl.SSLSupport.checkPemProviderLoaded;
 
 public class WebServerComponent implements ExternalComponent, WebServerComponentMarker {
 
@@ -64,6 +81,14 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
    // this should match the value of <display-name> in the console war's WEB-INF/web.xml
    public static final String WEB_CONSOLE_DISPLAY_NAME = System.getProperty("org.apache.activemq.artemis.webConsoleDisplayName", "hawtio");
 
+   public static final boolean DEFAULT_SNI_HOST_CHECK_VALUE = true;
+
+   public static final boolean DEFAULT_SNI_REQUIRED_VALUE = false;
+
+   public static final boolean DEFAULT_SSL_AUTO_RELOAD_VALUE = false;
+
+   public static final int DEFAULT_SCAN_PERIOD_VALUE = 5;
+
    private Server server;
    private HandlerList handlers;
    private WebServerDTO webServerConfig;
@@ -76,12 +101,23 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
    private String artemisInstance;
    private String artemisHome;
 
+   private int scanPeriod;
+   private Scanner scanner;
+   private ScheduledExecutorScheduler scannerScheduler;
+   private Map<String, List<Runnable>> scannerTasks = new HashMap<>();
+
    @Override
    public void configure(ComponentDTO config, String artemisInstance, String artemisHome) throws Exception {
       this.webServerConfig = (WebServerDTO) config;
       this.artemisInstance = artemisInstance;
       this.artemisHome = artemisHome;
 
+      if (webServerConfig.getScanPeriod() != null) {
+         scanPeriod = webServerConfig.getScanPeriod();
+      } else {
+         scanPeriod = DEFAULT_SCAN_PERIOD_VALUE;
+      }
+
       temporaryWarDir = Paths.get(artemisInstance != null ? artemisInstance : ".").resolve("tmp").resolve("webapps").toAbsolutePath();
       if (!Files.exists(temporaryWarDir)) {
          Files.createDirectories(temporaryWarDir);
@@ -95,20 +131,20 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
       }
       ActiveMQWebLogger.LOGGER.startingEmbeddedWebServer();
 
-      server = new Server();
+      server = new Server(new QueuedThreadPool(webServerConfig.maxThreads, webServerConfig.minThreads, webServerConfig.idleThreadTimeout));
       handlers = new HandlerList();
 
       HttpConfiguration httpConfiguration = new HttpConfiguration();
 
       if (this.webServerConfig.customizer != null) {
          try {
-            httpConfiguration.addCustomizer((HttpConfiguration.Customizer) Class.forName(this.webServerConfig.customizer).getConstructor().newInstance());
+            httpConfiguration.addCustomizer((HttpConfiguration.Customizer) ClassloadingUtil.getInstanceWithTypeCheck(this.webServerConfig.customizer, HttpConfiguration.Customizer.class, this.getClass().getClassLoader()));
          } catch (Throwable t) {
             ActiveMQWebLogger.LOGGER.customizerNotLoaded(this.webServerConfig.customizer, t);
          }
       }
 
-      List<BindingDTO> bindings = this.webServerConfig.getBindings();
+      List<BindingDTO> bindings = this.webServerConfig.getAllBindings();
       connectors = new ServerConnector[bindings.size()];
       String[] virtualHosts = new String[bindings.size()];
 
@@ -134,6 +170,20 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
                WebAppContext webContext = createWebAppContext(app.url, app.war, dirToUse, virtualHosts[i]);
                handlers.addHandler(webContext);
                webContext.setInitParameter(DIR_ALLOWED, "false");
+               webContext.getSessionHandler().getSessionCookieConfig().setComment("__SAME_SITE_STRICT__");
+               webContext.addEventListener(new ServletContextListener() {
+                  @Override
+                  public void contextInitialized(ServletContextEvent sce) {
+                     sce.getServletContext().addListener(new ServletRequestListener() {
+                        @Override
+                        public void requestDestroyed(ServletRequestEvent sre) {
+                           ServletRequestListener.super.requestDestroyed(sre);
+                           AuditLogger.currentCaller.remove();
+                           AuditLogger.remoteAddress.remove();
+                        }
+                     });
+                  }
+               });
                webContextData.add(new Pair(webContext, binding.uri));
             }
          }
@@ -169,8 +219,9 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
       defaultHandler.setServeIcon(false);
       defaultHandler.setRootRedirectLocation(this.webServerConfig.rootRedirectLocation);
 
-      if (this.webServerConfig.requestLog != null) {
-         handlers.addHandler(getLogHandler());
+      if (this.webServerConfig.requestLog != null &&
+         this.webServerConfig.requestLog.filename != null) {
+         server.setRequestLog(getRequestLog());
       }
 
       if (this.webServerConfig.webContentEnabled != null &&
@@ -220,6 +271,10 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
       if ("https".equals(scheme)) {
          SslContextFactory.Server sslFactory = new SslContextFactory.Server();
          sslFactory.setKeyStorePath(binding.keyStorePath == null ? artemisInstance + "/etc/keystore.jks" : binding.keyStorePath);
+         if (binding.keyStoreType != null) {
+            sslFactory.setKeyStoreType(binding.keyStoreType);
+            checkPemProviderLoaded(binding.keyStoreType);
+         }
          sslFactory.setKeyStorePassword(binding.getKeyStorePassword() == null ? "password" : binding.getKeyStorePassword());
 
          if (binding.getIncludedTLSProtocols() != null) {
@@ -239,17 +294,27 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
             if (binding.clientAuth) {
                sslFactory.setTrustStorePath(binding.trustStorePath);
                sslFactory.setTrustStorePassword(binding.getTrustStorePassword());
+               if (binding.trustStoreType != null) {
+                  sslFactory.setTrustStoreType(binding.trustStoreType);
+                  checkPemProviderLoaded(binding.trustStoreType);
+               }
             }
          }
+         if (Boolean.TRUE.equals(binding.getSslAutoReload())) {
+            addStoreResourceScannerTask(binding.getKeyStorePath(), binding.getKeyStoreType(), sslFactory);
+            addStoreResourceScannerTask(binding.getTrustStorePath(), binding.getTrustStoreType(), sslFactory);
+         }
 
          SslConnectionFactory sslConnectionFactory = new SslConnectionFactory(sslFactory, "HTTP/1.1");
 
-         httpConfiguration.addCustomizer(new SecureRequestCustomizer());
+         SecureRequestCustomizer secureRequestCustomizer = new SecureRequestCustomizer();
+         secureRequestCustomizer.setSniHostCheck(binding.getSniHostCheck() != null ? binding.getSniHostCheck() : DEFAULT_SNI_HOST_CHECK_VALUE);
+         secureRequestCustomizer.setSniRequired(binding.getSniRequired() != null ? binding.getSniRequired() : DEFAULT_SNI_REQUIRED_VALUE);
+         httpConfiguration.addCustomizer(secureRequestCustomizer);
          httpConfiguration.setSendServerVersion(false);
          HttpConnectionFactory httpFactory = new HttpConnectionFactory(httpConfiguration);
 
          connector = new ServerConnector(server, sslConnectionFactory, httpFactory);
-
       } else {
          httpConfiguration.setSendServerVersion(false);
          ConnectionFactory connectionFactory = new HttpConnectionFactory(httpConfiguration);
@@ -261,7 +326,93 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
       return connector;
    }
 
-   private RequestLogHandler getLogHandler() {
+   private File getStoreFile(String storeFilename) {
+      File storeFile = new File(storeFilename);
+      if (!storeFile.exists())
+         throw new IllegalArgumentException("Store file does not exist: " + storeFilename);
+      if (storeFile.isDirectory())
+         throw new IllegalArgumentException("Expected store file not directory: " + storeFilename);
+
+      return storeFile;
+   }
+
+   private File getParentStoreFile(File storeFile) {
+      File parentFile = storeFile.getParentFile();
+      if (!parentFile.exists() || !parentFile.isDirectory())
+         throw new IllegalArgumentException("Error obtaining store dir for " + storeFile);
+
+      return parentFile;
+   }
+
+   private Scanner getScanner() {
+      if (scannerScheduler == null) {
+         scannerScheduler = new ScheduledExecutorScheduler("WebScannerScheduler", true, 1);
+         server.addBean(scannerScheduler);
+      }
+
+      if (scanner == null) {
+         scanner = new Scanner(scannerScheduler);
+         scanner.setScanInterval(scanPeriod);
+         scanner.setReportDirs(false);
+         scanner.setReportExistingFilesOnStartup(false);
+         scanner.setScanDepth(1);
+         scanner.addListener((Scanner.BulkListener) filenames -> {
+            for (String filename: filenames) {
+               List<Runnable> tasks = scannerTasks.get(filename);
+               if (tasks != null) {
+                  tasks.forEach(t -> t.run());
+               }
+            }
+         });
+         server.addBean(scanner);
+      }
+
+      return scanner;
+   }
+
+   private void addScannerTask(File file, Runnable task) {
+      File parentFile = getParentStoreFile(file);
+      String storeFilename = file.toPath().toString();
+      List<Runnable> tasks = scannerTasks.get(storeFilename);
+      if (tasks == null) {
+         tasks = new ArrayList<>();
+         scannerTasks.put(storeFilename, tasks);
+      }
+      tasks.add(task);
+      getScanner().addDirectory(parentFile.toPath());
+   }
+
+   private void addStoreResourceScannerTask(String storeFilename, String storeType, SslContextFactory.Server sslFactory) {
+      if (storeFilename != null) {
+         File storeFile = getStoreFile(storeFilename);
+         addScannerTask(storeFile, () -> {
+            try {
+               sslFactory.reload(f -> { });
+            } catch (Exception e) {
+               logger.warn("Failed to reload the ssl factory related to {}", storeFile, e);
+            }
+         });
+
+         if (PemConfigUtil.isPemConfigStoreType(storeType)) {
+            String[] sources;
+
+            try (InputStream pemConfigStream = new FileInputStream(storeFile)) {
+               sources = PemConfigUtil.parseSources(pemConfigStream);
+            } catch (IOException e) {
+               throw new IllegalArgumentException("Invalid PEM Config file: " + e);
+            }
+
+            if (sources != null) {
+               for (String source : sources) {
+                  addStoreResourceScannerTask(source, null, sslFactory);
+               }
+            }
+         }
+
+      }
+   }
+
+   private RequestLog getRequestLog() {
       RequestLogWriter requestLogWriter = new RequestLogWriter();
       CustomRequestLog requestLog;
 
@@ -297,9 +448,7 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
          requestLog.setIgnorePaths(ignorePaths);
       }
 
-      RequestLogHandler requestLogHandler = new RequestLogHandler();
-      requestLogHandler.setRequestLog(requestLog);
-      return requestLogHandler;
+      return requestLog;
    }
 
    private File getLibFolder() {
@@ -371,7 +520,9 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
 
       webapp.setWar(warDirectory.resolve(warFile).toString());
 
-      webapp.setAttribute("org.eclipse.jetty.webapp.basetempdir", temporaryWarDir.toFile().getAbsolutePath());
+      String baseTempDir = temporaryWarDir.toFile().getAbsolutePath();
+      webapp.setAttribute("org.eclipse.jetty.webapp.basetempdir", baseTempDir);
+      webapp.setTempDirectory(new File(baseTempDir + File.separator + warFile));
 
       // Set the default authenticator factory to avoid NPE due to the following commit:
       // https://github.com/eclipse/jetty.project/commit/7e91d34177a880ecbe70009e8f200d02e3a0c5dd
@@ -393,6 +544,9 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
          ActiveMQWebLogger.LOGGER.stoppingEmbeddedWebServer();
          server.stop();
          server = null;
+         scanner = null;
+         scannerScheduler = null;
+         scannerTasks.clear();
          cleanupWebTemporaryFiles(webContextData);
          webContextData.clear();
          jolokiaUrls.clear();
@@ -405,4 +559,8 @@ public class WebServerComponent implements ExternalComponent, WebServerComponent
    public List<Pair<WebAppContext, String>> getWebContextData() {
       return this.webContextData;
    }
+
+   public Server getWebServer() {
+      return server;
+   }
 }
