@@ -20,6 +20,9 @@ import javax.jms.BytesMessage;
 import javax.jms.Connection;
 import javax.jms.ConnectionFactory;
 import javax.jms.DeliveryMode;
+import javax.jms.JMSConsumer;
+import javax.jms.JMSContext;
+import javax.jms.JMSException;
 import javax.jms.MapMessage;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
@@ -28,17 +31,23 @@ import javax.jms.Session;
 import javax.jms.StreamMessage;
 import javax.jms.TextMessage;
 import java.io.Serializable;
+import java.lang.invoke.MethodHandles;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Set;
+import java.util.concurrent.BrokenBarrierException;
 import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.CyclicBarrier;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicLongArray;
+import java.util.stream.Stream;
 
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ActiveMQIllegalStateException;
 import org.apache.activemq.artemis.api.core.Interceptor;
 import org.apache.activemq.artemis.api.core.Message;
+import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.client.ClientConsumer;
@@ -54,22 +63,40 @@ import org.apache.activemq.artemis.core.protocol.core.Packet;
 import org.apache.activemq.artemis.core.protocol.core.impl.PacketImpl;
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.Queue;
+import org.apache.activemq.artemis.core.server.ServerConsumer;
+import org.apache.activemq.artemis.core.server.ServerSession;
 import org.apache.activemq.artemis.jms.client.ActiveMQConnectionFactory;
+import org.apache.activemq.artemis.protocol.amqp.proton.AMQPSessionContext;
+import org.apache.activemq.artemis.protocol.amqp.proton.ProtonServerSenderContext;
 import org.apache.activemq.artemis.spi.core.protocol.RemotingConnection;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
 import org.apache.activemq.artemis.tests.util.Wait;
 import org.apache.activemq.artemis.utils.ByteUtil;
-import org.apache.activemq.artemis.utils.ConcurrentHashSet;
+import org.apache.activemq.artemis.utils.collections.ConcurrentHashSet;
 import org.apache.qpid.jms.JmsConnectionFactory;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 @RunWith(value = Parameterized.class)
 public class ConsumerTest extends ActiveMQTestBase {
 
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
+
+   @Override
+   protected boolean waitForBindings(ActiveMQServer server,
+                                     String address,
+                                     boolean local,
+                                     int expectedBindingCount,
+                                     int expectedConsumerCount,
+                                     long timeout) throws Exception {
+      return super.waitForBindings(server, address, local, expectedBindingCount, expectedConsumerCount, timeout);
+   }
+
    @Parameterized.Parameters(name = "isNetty={0}, persistent={1}")
    public static Collection getParameters() {
       return Arrays.asList(new Object[][]{{true, true}, {false, false}, {false, true}, {true, false}});
@@ -99,6 +126,8 @@ public class ConsumerTest extends ActiveMQTestBase {
 
       server = createServer(durable, isNetty());
 
+      server.getConfiguration().setAddressQueueScanPeriod(10);
+
       server.start();
 
       locator = createFactory(isNetty());
@@ -111,9 +140,11 @@ public class ConsumerTest extends ActiveMQTestBase {
 
       ClientSessionFactory sf = createSessionFactory(locator);
 
+      ClientSessionFactory sf2 = createSessionFactory(locator);
+
       ClientSession session = sf.createSession(false, true, true, true);
 
-      server.createQueue(QUEUE, RoutingType.ANYCAST, QUEUE, null, true, false);
+      server.createQueue(new QueueConfiguration(QUEUE).setRoutingType(RoutingType.ANYCAST));
 
       session.close();
 
@@ -126,7 +157,7 @@ public class ConsumerTest extends ActiveMQTestBase {
    public void testStressConnection() throws Exception {
 
       for (int i = 0; i < 10; i++) {
-         ServerLocator locatorSendx = createFactory(isNetty()).setReconnectAttempts(-1);
+         ServerLocator locatorSendx = createFactory(isNetty()).setReconnectAttempts(15);
          ClientSessionFactory factoryx = locatorSendx.createSessionFactory();
          factoryx.close();
          locatorSendx.close();
@@ -178,13 +209,13 @@ public class ConsumerTest extends ActiveMQTestBase {
 
       Assert.assertNotNull(message2);
 
-      System.out.println("Id::" + message2.getMessageID());
+      logger.debug("Id::{}", message2.getMessageID());
 
-      System.out.println("Received " + message2);
+      logger.debug("Received {}", message2);
 
-      System.out.println("Clie:" + ByteUtil.bytesToHex(message2.getBuffer().array(), 4));
+      logger.debug("Clie:{}", ByteUtil.bytesToHex(message2.getBuffer().array(), 4));
 
-      System.out.println("String::" + message2.getReadOnlyBodyBuffer().readString());
+      logger.debug("String::{}", message2.getReadOnlyBodyBuffer().readString());
 
       Assert.assertEquals("elo", message2.getStringProperty("hello"));
 
@@ -228,6 +259,161 @@ public class ConsumerTest extends ActiveMQTestBase {
       internalSend(2, 1);
    }
 
+   @Test
+   public void testAutoCreateMulticastAddress() throws Throwable {
+      if (!isNetty()) {
+         // no need to run the test, there's no AMQP support
+         return;
+      }
+
+      assertNull(server.getAddressInfo(SimpleString.toSimpleString("topic")));
+
+      ConnectionFactory factorySend = createFactory(2);
+      Connection connection = factorySend.createConnection();
+
+      try {
+         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+         javax.jms.Topic topic = session.createTopic("topic");
+         MessageProducer producer = session.createProducer(topic);
+         producer.setDeliveryMode(DeliveryMode.PERSISTENT);
+
+         TextMessage msg = session.createTextMessage("hello");
+         msg.setIntProperty("mycount", 0);
+         producer.send(msg);
+      } finally {
+         connection.close();
+      }
+
+      assertNotNull(server.getAddressInfo(SimpleString.toSimpleString("topic")));
+      assertEquals(RoutingType.MULTICAST, server.getAddressInfo(SimpleString.toSimpleString("topic")).getRoutingType());
+      assertEquals(0, server.getTotalMessageCount());
+   }
+
+   @Test
+   public void testAutoCreateCOnConsumerAMQP() throws Throwable {
+      testAutoCreate(2);
+   }
+
+   @Test
+   public void testAutoCreateCOnConsumerCore() throws Throwable {
+      testAutoCreate(1);
+   }
+
+   @Test
+   public void testAutoCreateCOnConsumerOpenWire() throws Throwable {
+      testAutoCreate(3);
+   }
+
+   private void testAutoCreate(int protocol) throws Throwable {
+
+      final SimpleString thisQueue = SimpleString.toSimpleString("ThisQueue");
+      if (!isNetty()) {
+         // no need to run the test, there's no AMQP support
+         return;
+      }
+
+      for (int i = 0; i < 10; i++) {
+         ConnectionFactory factorySend = createFactory(protocol);
+         Connection connection = factorySend.createConnection();
+
+         try {
+            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            javax.jms.Queue queue = session.createQueue(thisQueue.toString());
+            MessageProducer producer = session.createProducer(queue);
+
+            MessageConsumer consumer = session.createConsumer(queue);
+            connection.start();
+
+            producer.send(session.createTextMessage("hello"));
+
+            Assert.assertNotNull(consumer.receive(5000));
+            consumer.close();
+            session.close();
+         } finally {
+            connection.close();
+         }
+
+         Wait.waitFor(() -> server.getAddressInfo(thisQueue) == null, 1000, 10);
+         assertNull(server.getAddressInfo(thisQueue));
+         assertEquals(0, server.getTotalMessageCount());
+      }
+   }
+
+   @Test
+   public void testContextOnConsumerAMQP() throws Throwable {
+      if (!isNetty()) {
+         // no need to run the test, there's no AMQP support
+         return;
+      }
+
+      assertNull(server.getAddressInfo(SimpleString.toSimpleString("queue")));
+
+      ConnectionFactory factory = createFactory(2);
+      JMSContext context = factory.createContext("admin", "admin", Session.AUTO_ACKNOWLEDGE);
+
+      try {
+         javax.jms.Queue queue = context.createQueue("queue");
+
+         JMSConsumer consumer = context.createConsumer(queue);
+
+         ServerConsumer serverConsumer = null;
+         for (ServerSession session : server.getSessions()) {
+            for (ServerConsumer sessionConsumer : session.getServerConsumers()) {
+               serverConsumer = sessionConsumer;
+            }
+         }
+
+         Assert.assertTrue(serverConsumer.getProtocolContext() instanceof ProtonServerSenderContext);
+
+         final AMQPSessionContext sessionContext = ((ProtonServerSenderContext)
+            serverConsumer.getProtocolContext()).getSessionContext();
+
+         consumer.close();
+         final ServerConsumer lambdaServerConsumer = serverConsumer;
+         Wait.assertTrue(() -> lambdaServerConsumer.getProtocolContext() == null);
+
+         Wait.assertEquals(0, () -> sessionContext.getSenderCount(), 1000, 10);
+
+      } finally {
+         context.stop();
+         context.close();
+      }
+   }
+
+   @Test
+   public void testAutoDeleteAutoCreatedAddressAndQueue() throws Throwable {
+      if (!isNetty()) {
+         // no need to run the test, there's no AMQP support
+         return;
+      }
+
+      assertNull(server.getAddressInfo(SimpleString.toSimpleString("queue")));
+
+      ConnectionFactory factorySend = createFactory(2);
+      Connection connection = factorySend.createConnection();
+
+      try {
+         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+         javax.jms.Queue queue = session.createQueue("queue");
+         MessageProducer producer = session.createProducer(queue);
+         producer.setDeliveryMode(DeliveryMode.PERSISTENT);
+
+         TextMessage msg = session.createTextMessage("hello");
+         msg.setIntProperty("mycount", 0);
+         producer.send(msg);
+
+         connection.start();
+         MessageConsumer consumer = session.createConsumer(queue);
+         assertNotNull(consumer.receive(1000));
+      } finally {
+         connection.close();
+      }
+
+      Wait.assertTrue(() -> server.getAddressInfo(SimpleString.toSimpleString("queue")) == null);
+      Wait.assertTrue(() -> server.locateQueue(SimpleString.toSimpleString("queue")) == null);
+      Wait.assertEquals(0, server::getTotalMessageCount);
+   }
+
    @Test
    public void testSendCoreReceiveAMQP() throws Throwable {
 
@@ -300,7 +486,10 @@ public class ConsumerTest extends ActiveMQTestBase {
 
    private ConnectionFactory createFactory(int protocol) {
       switch (protocol) {
-         case 1: return new ActiveMQConnectionFactory();// core protocol
+         case 1: ActiveMQConnectionFactory coreCF = new ActiveMQConnectionFactory();// core protocol
+            coreCF.setCompressLargeMessage(true);
+            coreCF.setMinLargeMessageSize(10 * 1024);
+            return coreCF;
          case 2: return new JmsConnectionFactory("amqp://localhost:61616"); // amqp
          case 3: return new org.apache.activemq.ActiveMQConnectionFactory("tcp://localhost:61616"); // openwire
          default: return null;
@@ -308,8 +497,64 @@ public class ConsumerTest extends ActiveMQTestBase {
    }
 
 
+   public void internalSimpleSend(int protocolSender, int protocolConsumer) throws Throwable {
+
+      ConnectionFactory factorySend = createFactory(protocolSender);
+      ConnectionFactory factoryConsume = protocolConsumer == protocolSender ? factorySend : createFactory(protocolConsumer);
+
+      StringBuilder bufferLarge = new StringBuilder();
+      while (bufferLarge.length() < 100 * 1024) {
+         bufferLarge.append("          ");
+      }
+      final String bufferLargeContent = bufferLarge.toString();
+
+      try (Connection connection = factorySend.createConnection()) {
+         connection.start();
+         try (Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE)) {
+            javax.jms.Queue queue = session.createQueue(QUEUE.toString());
+            try (MessageProducer producer = session.createProducer(queue)) {
+               producer.setDeliveryMode(DeliveryMode.PERSISTENT);
+
+               TextMessage msg = session.createTextMessage("hello");
+               msg.setIntProperty("mycount", 0);
+               producer.send(msg);
+
+               msg = session.createTextMessage(bufferLargeContent);
+               msg.setIntProperty("mycount", 1);
+               producer.send(msg);
+            }
+         }
+
+      }
+      try (Connection connection = factoryConsume.createConnection()) {
+         connection.start();
+         try (Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE)) {
+            javax.jms.Queue queue = session.createQueue(QUEUE.toString());
+
+            try (MessageConsumer consumer = session.createConsumer(queue)) {
+
+               TextMessage message = (TextMessage) consumer.receive(1000);
+               Assert.assertNotNull(message);
+               Assert.assertEquals(0, message.getIntProperty("mycount"));
+               Assert.assertEquals("hello", message.getText());
+
+               message = (TextMessage) consumer.receive(1000);
+               Assert.assertNotNull(message);
+               Assert.assertEquals(1, message.getIntProperty("mycount"));
+               Assert.assertEquals(bufferLargeContent, message.getText());
+
+               Wait.waitFor(() -> server.getPagingManager().getGlobalSize() == 0, 5000, 100);
+               Assert.assertEquals(0, server.getPagingManager().getGlobalSize());
+            }
+         }
+      }
+   }
+
+
    public void internalSend(int protocolSender, int protocolConsumer) throws Throwable {
 
+      internalSimpleSend(protocolSender, protocolConsumer);
+
       ConnectionFactory factorySend = createFactory(protocolSender);
       ConnectionFactory factoryConsume = protocolConsumer == protocolSender ? factorySend : createFactory(protocolConsumer);
 
@@ -320,10 +565,16 @@ public class ConsumerTest extends ActiveMQTestBase {
          Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
          javax.jms.Queue queue = session.createQueue(QUEUE.toString());
          MessageProducer producer = session.createProducer(queue);
-         producer.setDeliveryMode(DeliveryMode.PERSISTENT);
+
+         if (durable) {
+            producer.setDeliveryMode(DeliveryMode.PERSISTENT);
+         } else {
+
+            producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
+         }
 
          long time = System.currentTimeMillis();
-         int NUMBER_OF_MESSAGES = 100;
+         int NUMBER_OF_MESSAGES = durable ? 5 : 50;
          for (int i = 0; i < NUMBER_OF_MESSAGES; i++) {
             TextMessage msg = session.createTextMessage("hello " + i);
             msg.setIntProperty("mycount", i);
@@ -348,7 +599,7 @@ public class ConsumerTest extends ActiveMQTestBase {
          }
          long end = System.currentTimeMillis();
 
-         System.out.println("Time = " + (end - time));
+         logger.debug("Time = {}", (end - time));
 
          {
             TextMessage dummyMessage = session.createTextMessage();
@@ -414,6 +665,19 @@ public class ConsumerTest extends ActiveMQTestBase {
          TextMessage msg = (TextMessage) consumer.receive(1000);
          Assert.assertEquals("testSelectorExampleFromSpecs:2", msg.getText());
 
+         consumer.close();
+
+         consumer = session.createConsumer(queue);
+         msg = (TextMessage)consumer.receive(5000);
+         Assert.assertNotNull(msg);
+
+         Assert.assertNull(consumer.receiveNoWait());
+
+         Wait.waitFor(() -> server.getPagingManager().getGlobalSize() == 0, 5000, 100);
+
+
+         Assert.assertEquals(0, server.getPagingManager().getGlobalSize());
+
       } finally {
          connection.close();
       }
@@ -697,10 +961,10 @@ public class ConsumerTest extends ActiveMQTestBase {
 
          sessions.add(session);
 
-         session.createQueue(QUEUE, QUEUE.concat("" + i), null, true);
+         session.createQueue(new QueueConfiguration(QUEUE.concat("" + i)).setAddress(QUEUE).setDurable(true));
 
          if (i == 0) {
-            session.createQueue(QUEUE_RESPONSE, QUEUE_RESPONSE);
+            session.createQueue(new QueueConfiguration(QUEUE_RESPONSE));
          }
 
          ClientConsumer consumer = session.createConsumer(QUEUE.concat("" + i));
@@ -712,7 +976,7 @@ public class ConsumerTest extends ActiveMQTestBase {
                @Override
                public void onMessage(final ClientMessage msg) {
                   try {
-                     ServerLocator locatorSendx = createFactory(isNetty()).setReconnectAttempts(-1);
+                     ServerLocator locatorSendx = createFactory(isNetty()).setReconnectAttempts(15);
                      ClientSessionFactory factoryx = locatorSendx.createSessionFactory();
                      ClientSession sessionSend = factoryx.createSession(true, true);
 
@@ -768,7 +1032,7 @@ public class ConsumerTest extends ActiveMQTestBase {
                }
 
                if (cons.receiveImmediate() != null) {
-                  System.out.println("ERROR: Received an extra message");
+                  logger.debug("ERROR: Received an extra message");
                   errors.incrementAndGet();
                }
                sessionSend.close();
@@ -916,4 +1180,148 @@ public class ConsumerTest extends ActiveMQTestBase {
       session.close();
    }
 
+   @Test
+   public void testMultipleConsumersOnSharedQueue() throws Throwable {
+      if (!isNetty() || this.durable) {
+         return;
+      }
+      final boolean durable = false;
+      final long TIMEOUT_MILLIS = TimeUnit.MINUTES.toMillis(1);
+      final int forks = 100;
+      final int queues = forks;
+      final int runs = 1;
+      final int messages = 1;
+      final ConnectionFactory factorySend = createFactory(1);
+      final AtomicLongArray receivedMessages = new AtomicLongArray(forks);
+      final Thread[] producersRunners = new Thread[forks];
+      final Thread[] consumersRunners = new Thread[forks];
+      //parties are forks (1 producer 1 consumer) + 1 controller in the main test thread
+      final CyclicBarrier onStartRun = new CyclicBarrier((forks * 2) + 1);
+      final CyclicBarrier onFinishRun = new CyclicBarrier((forks * 2) + 1);
+
+      final int messagesSent = forks * messages;
+      final AtomicInteger messagesRecieved = new AtomicInteger(0);
+
+      for (int i = 0; i < forks; i++) {
+         final int forkIndex = i;
+         final String queueName = "q_" + (forkIndex % queues);
+         final Thread producerRunner = new Thread(() -> {
+            try (Connection connection = factorySend.createConnection()) {
+               connection.start();
+               try (Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE)) {
+                  final javax.jms.Queue queue = session.createQueue(queueName);
+                  try (MessageProducer producer = session.createProducer(queue)) {
+                     producer.setDeliveryMode(durable ? DeliveryMode.PERSISTENT : DeliveryMode.NON_PERSISTENT);
+                     for (int r = 0; r < runs; r++) {
+                        onStartRun.await();
+                        for (int m = 0; m < messages; m++) {
+                           final BytesMessage bytesMessage = session.createBytesMessage();
+                           bytesMessage.writeInt(forkIndex);
+                           producer.send(bytesMessage);
+                        }
+                        onFinishRun.await();
+                     }
+                  } catch (InterruptedException | BrokenBarrierException e) {
+                     e.printStackTrace();
+                  }
+               }
+            } catch (JMSException e) {
+               e.printStackTrace();
+            }
+         });
+
+         producerRunner.setDaemon(true);
+
+         final Thread consumerRunner = new Thread(() -> {
+            try (Connection connection = factorySend.createConnection()) {
+               connection.start();
+               try (Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE)) {
+                  final javax.jms.Queue queue = session.createQueue(queueName);
+                  try (MessageConsumer consumer = session.createConsumer(queue)) {
+                     for (int r = 0; r < runs; r++) {
+                        onStartRun.await();
+                        while (messagesRecieved.get() != messagesSent) {
+                           final BytesMessage receivedMessage = (BytesMessage) consumer.receive(1000);
+                           if (receivedMessage != null) {
+                              final int receivedConsumerIndex = receivedMessage.readInt();
+                              receivedMessages.getAndIncrement(receivedConsumerIndex);
+                              messagesRecieved.incrementAndGet();
+                           }
+                        }
+                        onFinishRun.await();
+                     }
+                  } catch (InterruptedException e) {
+                     e.printStackTrace();
+                  } catch (BrokenBarrierException e) {
+                     e.printStackTrace();
+                  }
+               }
+            } catch (JMSException e) {
+               e.printStackTrace();
+            }
+         });
+         consumerRunner.setDaemon(true);
+         consumersRunners[forkIndex] = consumerRunner;
+         producersRunners[forkIndex] = producerRunner;
+      }
+      Stream.of(consumersRunners).forEach(Thread::start);
+      Stream.of(producersRunners).forEach(Thread::start);
+      final long messagesPerRun = (forks * messages);
+      for (int r = 0; r < runs; r++) {
+         onStartRun.await(TIMEOUT_MILLIS, TimeUnit.MILLISECONDS);
+         logger.debug("started run {}", r);
+         final long start = System.currentTimeMillis();
+         onFinishRun.await(TIMEOUT_MILLIS, TimeUnit.MILLISECONDS);
+         final long elapsedMillis = System.currentTimeMillis() - start;
+         logger.debug("{} msg/sec", (messagesPerRun * 1000L) / elapsedMillis);
+      }
+      Stream.of(producersRunners).forEach(runner -> {
+         try {
+            runner.join(TIMEOUT_MILLIS * runs);
+         } catch (InterruptedException e) {
+            e.printStackTrace();
+         }
+      });
+      Stream.of(producersRunners).forEach(Thread::interrupt);
+      Stream.of(consumersRunners).forEach(runner -> {
+         try {
+            runner.join(TIMEOUT_MILLIS * runs);
+         } catch (InterruptedException e) {
+            e.printStackTrace();
+         }
+      });
+      Stream.of(consumersRunners).forEach(Thread::interrupt);
+      for (int i = 0; i < forks; i++) {
+         Assert.assertEquals("The consumer " + i + " must receive all the messages sent.", messages * runs, receivedMessages.get(i));
+      }
+   }
+
+   @Test
+   public void testConsumerXpathSelector() throws Exception {
+      final SimpleString BODY = SimpleString.toSimpleString("<root><a key='first' num='1'/><b key='second' num='2'>b</b></root>");
+      ClientSessionFactory sf = createSessionFactory(locator);
+
+      ClientSession session = sf.createSession(false, true, false, true);
+
+      ClientProducer producer = session.createProducer(QUEUE);
+
+      ClientMessage message = session.createMessage(false);
+      message.setType(Message.TEXT_TYPE);
+      message.getBodyBuffer().writeNullableSimpleString(SimpleString.toSimpleString("wrong"));
+      producer.send(message);
+      message = session.createMessage(false);
+      message.setType(Message.TEXT_TYPE);
+      message.getBodyBuffer().writeNullableSimpleString(BODY);
+      producer.send(message);
+
+      ClientConsumer consumer = session.createConsumer(QUEUE.toString(), "XPATH 'root/a'");
+      session.start();
+      ClientMessage message2 = consumer.receive(5000);
+      Assert.assertNotNull(message2);
+
+      Assert.assertEquals(BODY, message2.getBodyBuffer().readNullableSimpleString());
+      Assert.assertEquals(1, getMessageCount(((Queue) server.getPostOffice().getBinding(QUEUE).getBindable())));
+
+      session.close();
+   }
 }
