@@ -48,6 +48,7 @@ import org.apache.activemq.artemis.api.core.client.ClientProducer;
 import org.apache.activemq.artemis.api.core.client.ClientSession;
 import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
 import org.apache.activemq.artemis.api.core.client.ServerLocator;
+import org.apache.activemq.artemis.api.core.management.BridgeControl;
 import org.apache.activemq.artemis.core.config.BridgeConfiguration;
 import org.apache.activemq.artemis.core.config.Configuration;
 import org.apache.activemq.artemis.core.config.TransformerConfiguration;
@@ -82,17 +83,19 @@ import org.apache.activemq.artemis.tests.util.Wait;
 import org.apache.activemq.artemis.utils.RandomUtil;
 import org.apache.activemq.artemis.utils.ReusableLatch;
 import org.apache.activemq.artemis.utils.collections.LinkedListIterator;
-import org.jboss.logging.Logger;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
 
 @RunWith(value = Parameterized.class)
 public class BridgeTest extends ActiveMQTestBase {
 
-   private static final Logger log = Logger.getLogger(BridgeTest.class);
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
    private ActiveMQServer server0;
    private ActiveMQServer server1;
@@ -1494,7 +1497,7 @@ public class BridgeTest extends ActiveMQTestBase {
             msgCount.incrementAndGet();
 
             if (i % 500 == 0)
-               instanceLog.debug("received " + i);
+               logger.debug("received {}", i);
          }
 
          boolean failed = false;
@@ -1505,7 +1508,7 @@ public class BridgeTest extends ActiveMQTestBase {
          }
 
          for (int i = 0; i < numMessages; i++) {
-            AtomicInteger msgCount = receivedMsg.get(Integer.valueOf(i));
+            AtomicInteger msgCount = receivedMsg.get(i);
             if (msgCount == null) {
                System.err.println("Msg " + i + " wasn't received");
                failed = true;
@@ -1812,7 +1815,7 @@ public class BridgeTest extends ActiveMQTestBase {
 
          File outputFile = new File(getTemporaryDir(), "huge_message_received.dat");
 
-         instanceLog.debug("-----message save to: " + outputFile.getAbsolutePath());
+         logger.debug("-----message save to: {}", outputFile.getAbsolutePath());
          FileOutputStream fileOutputStream = new FileOutputStream(outputFile);
 
          BufferedOutputStream bufferedOutput = new BufferedOutputStream(fileOutputStream);
@@ -1857,7 +1860,7 @@ public class BridgeTest extends ActiveMQTestBase {
 
       createFile(fileInput, largeMessageSize);
 
-      instanceLog.debug("File created at: " + fileInput.getAbsolutePath());
+      logger.debug("File created at: {}", fileInput.getAbsolutePath());
 
       ClientMessage message = session.createMessage(Message.BYTES_TYPE, true);
 
@@ -1871,13 +1874,13 @@ public class BridgeTest extends ActiveMQTestBase {
 
    private static void createFile(final File file, final long fileSize) throws IOException {
       if (file.exists()) {
-         log.warn("---file already there " + file.length());
+         logger.warn("---file already there {}", file.length());
          return;
       }
       FileOutputStream fileOut = new FileOutputStream(file);
       BufferedOutputStream buffOut = new BufferedOutputStream(fileOut);
       byte[] outBuffer = new byte[1024 * 1024];
-      log.debug(" --- creating file, size: " + fileSize);
+      logger.debug(" --- creating file, size: {}", fileSize);
       for (long i = 0; i < fileSize; i += outBuffer.length) {
          buffOut.write(outBuffer);
       }
@@ -2044,6 +2047,41 @@ public class BridgeTest extends ActiveMQTestBase {
       Assert.assertNotNull(consumer.receive(200));
    }
 
+   @Test
+   public void testManagementLeak() throws Exception {
+      final SimpleString ADDRESS = new SimpleString("myAddress");
+      final SimpleString QUEUE = new SimpleString("myQueue");
+      final SimpleString FORWARDING_ADDRESS = new SimpleString("myForwardingAddress");
+      final SimpleString FORWARDING_QUEUE = new SimpleString("myForwardingQueue");
+      final String BRIDGE = "myBridge";
+
+      ActiveMQServer server = addServer(new ActiveMQServerImpl(createDefaultConfig(0, isNetty()).addConnectorConfiguration("myConnector", new TransportConfiguration(getConnector()))));
+      server.start();
+      server.waitForActivation(100, TimeUnit.MILLISECONDS);
+      server.createQueue(new QueueConfiguration(QUEUE)
+                            .setAddress(ADDRESS)
+                            .setRoutingType(RoutingType.ANYCAST)
+                            .setDurable(false));
+      server.createQueue(new QueueConfiguration(FORWARDING_QUEUE)
+                            .setAddress(FORWARDING_ADDRESS)
+                            .setRoutingType(RoutingType.ANYCAST)
+                            .setDurable(false));
+      ArrayList<String> connectors = new ArrayList<>();
+      connectors.add("myConnector");
+      final int concurrency = 20;
+      BridgeConfiguration config = new BridgeConfiguration()
+         .setName(BRIDGE)
+         .setQueueName(QUEUE.toString())
+         .setForwardingAddress(FORWARDING_ADDRESS.toString())
+         .setStaticConnectors(connectors)
+         .setConcurrency(concurrency);
+      server.deployBridge(config);
+      assertEquals(concurrency, server.getManagementService().getResources(BridgeControl.class).length);
+      server.destroyBridge(config.getName());
+
+      assertEquals(0, server.getManagementService().getResources(BridgeControl.class).length);
+   }
+
    /**
     * It will inspect the journal directly and determine if there are queues on this journal,
     *
