@@ -252,6 +252,11 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (String) proxy.retrieveAttributeValue("lastValueKey");
          }
 
+         @Override
+         public boolean isInternalQueue() {
+            return (boolean) proxy.retrieveAttributeValue("internalQueue");
+         }
+
          @Override
          public int getConsumersBeforeDispatch() {
             return (Integer) proxy.retrieveAttributeValue("consumersBeforeDispatch");
@@ -415,6 +420,22 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (String) proxy.invokeOperation("getFirstMessageAsJSON");
          }
 
+         /**
+          * Returns the first message on the queue as JSON
+          */
+         @Override
+         public String peekFirstMessageAsJSON() throws Exception {
+            return (String) proxy.invokeOperation("peekFirstMessageAsJSON");
+         }
+
+         /**
+          * Returns the first scheduled message on the queue as JSON
+          */
+         @Override
+         public String peekFirstScheduledMessageAsJSON() throws Exception {
+            return (String) proxy.invokeOperation("peekFirstScheduledMessageAsJSON");
+         }
+
          /**
           * Returns the timestamp of the first message in milliseconds.
           */
@@ -545,7 +566,7 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
 
          @Override
          public int sendMessagesToDeadLetterAddress(final String filterStr) throws Exception {
-            return (Integer) proxy.invokeOperation("sendMessagesToDeadLetterAddress", filterStr);
+            return (Integer) proxy.invokeOperation(Integer.class, "sendMessagesToDeadLetterAddress", filterStr);
          }
 
          @Override
