@@ -16,6 +16,7 @@
  */
 package org.apache.activemq.artemis.tests.integration.server;
 
+import javax.management.MBeanServer;
 import java.lang.reflect.Field;
 import java.net.URI;
 import java.util.Arrays;
@@ -28,6 +29,7 @@ import java.util.concurrent.Executor;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.activemq.artemis.api.core.Message;
+import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.client.ActiveMQClient;
@@ -36,11 +38,12 @@ import org.apache.activemq.artemis.api.core.client.ClientMessage;
 import org.apache.activemq.artemis.api.core.client.ClientProducer;
 import org.apache.activemq.artemis.api.core.client.ClientSession;
 import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
+import org.apache.activemq.artemis.api.core.management.QueueControl;
 import org.apache.activemq.artemis.core.client.impl.ClientSessionFactoryImpl;
 import org.apache.activemq.artemis.core.client.impl.ServerLocatorImpl;
 import org.apache.activemq.artemis.core.client.impl.ServerLocatorInternal;
 import org.apache.activemq.artemis.core.config.ScaleDownConfiguration;
-import org.apache.activemq.artemis.core.config.ha.LiveOnlyPolicyConfiguration;
+import org.apache.activemq.artemis.core.config.ha.PrimaryOnlyPolicyConfiguration;
 import org.apache.activemq.artemis.core.postoffice.Binding;
 import org.apache.activemq.artemis.core.postoffice.impl.LocalQueueBinding;
 import org.apache.activemq.artemis.core.server.cluster.ClusterController;
@@ -48,6 +51,7 @@ import org.apache.activemq.artemis.core.server.cluster.impl.MessageLoadBalancing
 import org.apache.activemq.artemis.core.server.impl.QueueImpl;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.tests.integration.cluster.distribution.ClusterTestBase;
+import org.apache.activemq.artemis.tests.integration.management.ManagementControlHelper;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
 import org.apache.activemq.artemis.tests.util.Wait;
 import org.apache.activemq.transport.amqp.client.AmqpClient;
@@ -85,12 +89,12 @@ public class ScaleDownTest extends ClusterTestBase {
    @Before
    public void setUp() throws Exception {
       super.setUp();
-      setupLiveServer(0, isFileStorage(), isNetty(), true);
-      setupLiveServer(1, isFileStorage(), isNetty(), true);
+      setupPrimaryServer(0, isFileStorage(), isNetty(), true);
+      setupPrimaryServer(1, isFileStorage(), isNetty(), true);
       servers[0].getConfiguration().addAcceptorConfiguration("amqp", AMQP_ACCEPTOR_URI + "?protocols=AMQP");
-      LiveOnlyPolicyConfiguration haPolicyConfiguration0 = (LiveOnlyPolicyConfiguration) servers[0].getConfiguration().getHAPolicyConfiguration();
+      PrimaryOnlyPolicyConfiguration haPolicyConfiguration0 = (PrimaryOnlyPolicyConfiguration) servers[0].getConfiguration().getHAPolicyConfiguration();
       haPolicyConfiguration0.setScaleDownConfiguration(new ScaleDownConfiguration());
-      LiveOnlyPolicyConfiguration haPolicyConfiguration1 = (LiveOnlyPolicyConfiguration) servers[1].getConfiguration().getHAPolicyConfiguration();
+      PrimaryOnlyPolicyConfiguration haPolicyConfiguration1 = (PrimaryOnlyPolicyConfiguration) servers[1].getConfiguration().getHAPolicyConfiguration();
       haPolicyConfiguration1.setScaleDownConfiguration(new ScaleDownConfiguration());
       if (useScaleDownGroupName) {
          haPolicyConfiguration0.getScaleDownConfiguration().setGroupName("bill");
@@ -518,7 +522,7 @@ public class ScaleDownTest extends ClusterTestBase {
          ClientMessage msg = session.createMessage(true);
          msg.getBodyBuffer().writeString("Bob the giant pig " + i);
          msg.putBooleanProperty("myBooleanProperty", Boolean.TRUE);
-         msg.putByteProperty("myByteProperty", Byte.valueOf("0"));
+         msg.putByteProperty("myByteProperty", Byte.parseByte("0"));
          msg.putBytesProperty("myBytesProperty", new byte[]{0, 1, 2, 3, 4});
          msg.putDoubleProperty("myDoubleProperty", i * 1.6);
          msg.putFloatProperty("myFloatProperty", i * 2.5F);
@@ -796,4 +800,47 @@ public class ScaleDownTest extends ClusterTestBase {
 
       consumer.close();
    }
+
+   @Test
+   public void testScaleDownPagedMessageWithMultipleAutoCreatedDLAResources() throws Exception {
+      final SimpleString dla = new SimpleString("DLA");
+      final SimpleString qName = new SimpleString("Q");
+      final SimpleString adName = new SimpleString("ADDR");
+      final String sampleText = "Put me on DLQ";
+      final int messageCount = 10;
+      final int numQueues = 2;
+
+      AddressSettings addressSettings = new AddressSettings().setDeadLetterAddress(dla).setAutoCreateDeadLetterResources(true).setDeadLetterQueuePrefix(dla);
+      AddressSettings dlaAddressSettings = new AddressSettings().setDeadLetterAddress(dla).setMaxSizeBytes(200L).setAutoCreateQueues(true);
+
+      servers[0].getAddressSettingsRepository().addMatch( "#", addressSettings);
+      servers[0].getAddressSettingsRepository().addMatch(dla.toString(), dlaAddressSettings);
+
+      ClientSessionFactory sf = sfs[0];
+      ClientSession session = addClientSession(sf.createSession(true, true));
+      MBeanServer mbeanServer = servers[0].getMBeanServer();
+
+      for (int i = 0; i < numQueues; i++) {
+         SimpleString curAddr = adName.concat(String.valueOf(i));
+         SimpleString curQ = qName.concat(String.valueOf(i));
+         SimpleString dlq = dla.concat(curAddr);
+
+         session.createQueue(new QueueConfiguration(curQ).setAddress(curAddr).setDurable(true));
+         ClientProducer producer = session.createProducer(curAddr);
+
+         for (int p = 0; p < messageCount; p++) {
+            producer.send(createTextMessage(session, sampleText));
+         }
+
+         QueueControl queueControl = ManagementControlHelper.createQueueControl(curAddr, curQ, mbeanServer);
+         Assert.assertEquals(messageCount, queueControl.sendMessagesToDeadLetterAddress(null));
+         Assert.assertEquals(0, queueControl.getMessageCount());
+         Wait.assertTrue(servers[0].locateQueue(dlq).getPagingStore()::isPaging);
+      }
+
+      servers[0].getActiveMQServerControl().scaleDown(servers[0].getConfiguration().getClusterConfigurations().iterator().next().getStaticConnectors().get(0));
+      Wait.assertEquals((messageCount * numQueues), () -> servers[1].getTotalMessageCount());
+
+   }
+
 }
